@@ -4,47 +4,32 @@ import typing as tp
 import numpy as np
 
 
-from static_frame.core.series import Series
+from static_frame.core.container import ContainerBase
+from static_frame.core.display import Display
+from static_frame.core.display import DisplayActive
+from static_frame.core.display import DisplayConfig
+from static_frame.core.display import DisplayHeader
+from static_frame.core.doc_str import doc_inject
+from static_frame.core.exception import ErrorInitBus
 from static_frame.core.frame import Frame
-# from static_frame.core.frame import Index
-
+from static_frame.core.hloc import HLoc
+from static_frame.core.node_selector import InterfaceGetItem
+from static_frame.core.series import Series
 from static_frame.core.store import Store
-from static_frame.core.store_zip import StoreZipCSV
-from static_frame.core.store_zip import StoreZipTSV
-from static_frame.core.store_zip import StoreZipPickle
-from static_frame.core.store_xlsx import StoreXLSX
-from static_frame.core.store_sqlite import StoreSQLite
-from static_frame.core.store_hdf5 import StoreHDF5
-
-# from static_frame.core.store import StoreConfig
 from static_frame.core.store import StoreConfigMap
 from static_frame.core.store import StoreConfigMapInitializer
-
-
-from static_frame.core.exception import ErrorInitBus
-from static_frame.core.util import GetItemKeyType
-from static_frame.core.util import DTYPE_OBJECT
+from static_frame.core.store_hdf5 import StoreHDF5
+from static_frame.core.store_sqlite import StoreSQLite
+from static_frame.core.store_xlsx import StoreXLSX
+from static_frame.core.store_zip import StoreZipCSV
+from static_frame.core.store_zip import StoreZipPickle
+from static_frame.core.store_zip import StoreZipTSV
 from static_frame.core.util import DTYPE_BOOL
-# from static_frame.core.util import DTYPE_INT_DEFAULT
 from static_frame.core.util import DTYPE_FLOAT_DEFAULT
-from static_frame.core.util import PathSpecifier
+from static_frame.core.util import DTYPE_OBJECT
+from static_frame.core.util import GetItemKeyType
 from static_frame.core.util import NULL_SLICE
-# from static_frame.core.util import DtypesSpecifier
-
-from static_frame.core.selector_node import InterfaceGetItem
-
-from static_frame.core.hloc import HLoc
-
-from static_frame.core.display import DisplayConfig
-from static_frame.core.display import DisplayActive
-from static_frame.core.display import Display
-from static_frame.core.display import DisplayHeader
-
-from static_frame.core.doc_str import doc_inject
-
-from static_frame.core.container import ContainerBase
-
-from static_frame.core.selector_node import TContainer
+from static_frame.core.util import PathSpecifier
 
 
 #-------------------------------------------------------------------------------
@@ -58,7 +43,7 @@ class FrameDeferred(metaclass=FrameDefferedMeta):
     '''
 
 #-------------------------------------------------------------------------------
-class Bus(ContainerBase):
+class Bus(ContainerBase): # not a ContainerOperand
 
     __slots__ = (
         '_loaded',
@@ -211,8 +196,8 @@ class Bus(ContainerBase):
     # delegation
 
     def __getattr__(self, name: str) -> tp.Any:
-        if name == 'interface':
-            return getattr(self.__class__, 'interface')
+        # if name == 'interface':
+        #     return getattr(self.__class__, 'interface')
 
         try:
             return getattr(self._series, name)
@@ -273,11 +258,11 @@ class Bus(ContainerBase):
         values = self._series.values[key]
 
         if not isinstance(values, np.ndarray): # if we have a single element
-            return values
+            return values #type: ignore
         series = Series(
                 values,
                 index=self._series._index.iloc[key],
-                name=self._name)
+                name=self._series._name)
         return self.__class__(series=series,
                 store=self._store,
                 config=self._config,
@@ -298,12 +283,12 @@ class Bus(ContainerBase):
                 values = np.array(values)
                 values.flags.writeable = False
             else:
-                return values
+                return values #type: ignore
 
         series = Series(values,
                 index=self._series._index.iloc[iloc_key],
                 own_index=True,
-                name=self._name)
+                name=self._series._name)
         return self.__class__(series=series,
                 store=self._store,
                 config=self._config,
@@ -325,12 +310,12 @@ class Bus(ContainerBase):
     # interfaces
 
     @property
-    def loc(self) -> InterfaceGetItem[TContainer]:
-        return InterfaceGetItem(self._extract_loc)  #type: ignore
+    def loc(self) -> InterfaceGetItem['Bus']:
+        return InterfaceGetItem(self._extract_loc)
 
     @property
-    def iloc(self) -> InterfaceGetItem[TContainer]:
-        return InterfaceGetItem(self._extract_iloc)  #type: ignore
+    def iloc(self) -> InterfaceGetItem['Bus']:
+        return InterfaceGetItem(self._extract_iloc)
 
 
     # ---------------------------------------------------------------------------
@@ -368,29 +353,13 @@ class Bus(ContainerBase):
         Args:
             {config}
         '''
-        # NOTE: the key change is providing the Bus as the displayed class
-
+        # NOTE: the key change over serires is providing the Bus as the displayed class
         config = config or DisplayActive.get()
-
-        d = Display([],
-                config=config,
-                outermost=True,
-                index_depth=1,
-                header_depth=2) # series and index header
-
-        display_index = self._index.display(config=config)
-        d.extend_display(display_index)
-
-        d.extend_display(Display.from_values(
-                self._series.values, # do not force loading with self.values
-                header='',
-                config=config))
-
         display_cls = Display.from_values((),
                 header=DisplayHeader(self.__class__, self._series._name),
                 config=config)
-        d.insert_displays(display_cls.flatten())
-        return d
+        return self._series._display(config, display_cls)
+
 
     #---------------------------------------------------------------------------
     # extended disciptors
@@ -400,7 +369,7 @@ class Bus(ContainerBase):
         '''Returns a Series of tuples of dtypes, one for each loaded Frame.
         '''
         if not self._loaded.any():
-            return Series(None, index=self._series._index)
+            return Series.from_element(None, index=self._series._index)
 
         def gen() -> tp.Iterator[tp.Tuple[tp.Hashable, tp.Optional[tp.Tuple[int, ...]]]]:
             for label, f in zip(self._series._index, self._series.values):
@@ -465,6 +434,70 @@ class Bus(ContainerBase):
 
         return tp.cast(Frame, Frame.from_concat(gen(), axis=1))
 
+    #---------------------------------------------------------------------------
+    @doc_inject()
+    def equals(self,
+            other: tp.Any,
+            *,
+            compare_name: bool = False,
+            compare_dtype: bool = False,
+            compare_class: bool = False,
+            skipna: bool = True,
+            ) -> bool:
+        '''
+        {doc}
+
+        Args:
+            {compare_name}
+            {compare_dtype}
+            {compare_class}
+            {skipna}
+        '''
+
+        if id(other) == id(self):
+            return True
+
+        if compare_class and self.__class__ != other.__class__:
+            return False
+        elif not isinstance(other, Bus):
+            return False
+
+        # defer updating cache
+        self._update_series_cache_all()
+
+        if len(self._series) != len(other._series):
+            return False
+
+        if compare_name and self._series._name != other._series._name:
+            return False
+
+        # NOTE: dtype self._series is always object
+
+        if not self._series.index.equals(
+                other._series.index,
+                compare_name=compare_name,
+                compare_dtype=compare_dtype,
+                compare_class=compare_class,
+                skipna=skipna,
+                ):
+            return False
+
+        # can zip because length of Series already match
+        for (frame_self, frame_other) in zip(
+                self._series.values, other._series.values):
+            if not frame_self.equals(frame_other,
+                    compare_name=compare_name,
+                    compare_dtype=compare_dtype,
+                    compare_class=compare_class,
+                    skipna=skipna,
+                    ):
+                return False
+
+        return True
+
+
+
+
 
     #---------------------------------------------------------------------------
     # exporters
