@@ -9,30 +9,23 @@ from collections import namedtuple
 
 import numpy as np
 
-from static_frame.core.util import EMPTY_TUPLE
-from static_frame.core.util import AnyCallable
-from static_frame.core.util import DTYPE_INT_KIND
-from static_frame.core.util import DTYPE_STR_KIND
-from static_frame.core.util import DTYPE_NAN_KIND
-from static_frame.core.util import DTYPE_NAT_KIND
-from static_frame.core.util import isna_array
-
-# from static_frame.core.util import DTYPE_BOOL
-# from static_frame.core.util import DTYPE_FLOAT_DEFAULT
-
-from static_frame.core.util import DTYPES_BOOL
-from static_frame.core.util import DTYPES_INEXACT
-from static_frame.core.util import DTYPE_FLOAT_DEFAULT
-# from static_frame.core.util import NAT
-
-from static_frame.core.doc_str import DOC_TEMPLATE
 
 from static_frame.core.display import Display
-from static_frame.core.display import DisplayConfig
-# from static_frame.core.display import DisplayConfigs
 from static_frame.core.display import DisplayActive
-
+from static_frame.core.display import DisplayConfig
+from static_frame.core.display import DisplayFormats
 from static_frame.core.doc_str import doc_inject
+from static_frame.core.doc_str import DOC_TEMPLATE
+from static_frame.core.util import AnyCallable
+from static_frame.core.util import DTYPE_FLOAT_DEFAULT
+from static_frame.core.util import DTYPE_INT_KIND
+from static_frame.core.util import DTYPE_NAN_KIND
+from static_frame.core.util import DTYPE_NAT_KIND
+from static_frame.core.util import DTYPE_STR_KIND
+from static_frame.core.util import DTYPES_BOOL
+from static_frame.core.util import DTYPES_INEXACT
+from static_frame.core.util import EMPTY_TUPLE
+from static_frame.core.util import isna_array
 
 
 if tp.TYPE_CHECKING:
@@ -479,11 +472,11 @@ class ContainerOperandMeta(ContainerMeta):
         return func
 
 
-    def __new__(mcs,
+    def __new__(mcs, #type: ignore
             name: str,
             bases: tp.Tuple[type, ...],
             attrs: tp.Dict[str, object
-            ]) -> type:
+            ]) -> type: #must return a subtype of "ContainerOperandMeta"
         '''
         Create and assign all autopopulated functions. This __new__ is on the metaclass, not the class, and is thus only called once per class.
         '''
@@ -584,6 +577,7 @@ class ContainerOperand(ContainerBase, metaclass=ContainerOperandMeta):
     __slots__ = EMPTY_TUPLE
 
     interface: 'Frame' # property that returns a Frame
+    values: np.ndarray
 
     __pos__: tp.Callable[[T], T]
     __neg__: tp.Callable[[T], T]
@@ -654,3 +648,18 @@ class ContainerOperand(ContainerBase, metaclass=ContainerOperandMeta):
     def cumprod(self, axis: int = 0, skipna: bool = True) -> tp.Any:
         raise NotImplementedError() # pragma: no cover
 
+
+    def _repr_html_(self) -> str:
+        '''
+        Provide HTML representation for Jupyter Notebooks.
+        '''
+        # NOTE: We observe that Jupyter will window big content into scrollable component, so do not limit output and introduce ellipsis.
+
+        config = DisplayActive.get(
+                display_format=DisplayFormats.HTML_TABLE,
+                type_show=False,
+                display_columns=np.inf,
+                display_rows=np.inf,
+                )
+        # modify the active display to be for HTML
+        return repr(self.display(config))
\ No newline at end of file
