@@ -3,37 +3,44 @@ This module us for utilty functions that take as input and / or return Container
 '''
 
 from collections import defaultdict
+from itertools import zip_longest
 
 import numpy as np
+from numpy import char as npc
+
 import typing as tp
 
+
+from static_frame.core.index_base import IndexBase
+from static_frame.core.util import AnyCallable
+from static_frame.core.util import Bloc2DKeyType
+from static_frame.core.util import column_2d_filter
+from static_frame.core.util import DEFAULT_SORT_KIND
+from static_frame.core.util import DepthLevelSpecifier
+from static_frame.core.util import DTYPE_BOOL
+from static_frame.core.util import DTYPE_OBJECT
+from static_frame.core.util import DTYPE_STR
+from static_frame.core.util import DTYPE_STR_KIND
+from static_frame.core.util import DtypesSpecifier
+from static_frame.core.util import GetItemKeyType
+from static_frame.core.util import IndexConstructor
+from static_frame.core.util import IndexConstructors
+from static_frame.core.util import IndexInitializer
+from static_frame.core.util import iterable_to_array_1d
+from static_frame.core.util import NULL_SLICE
+from static_frame.core.util import slice_to_ascending_slice
+from static_frame.core.util import STATIC_ATTR
+from static_frame.core.util import UFunc
+
+
 if tp.TYPE_CHECKING:
     import pandas as pd #pylint: disable=W0611 #pragma: no cover
+    from static_frame.core.type_blocks import TypeBlocks #pylint: disable=W0611 #pragma: no cover
     from static_frame.core.series import Series #pylint: disable=W0611 #pragma: no cover
     from static_frame.core.frame import Frame #pylint: disable=W0611 #pragma: no cover
     from static_frame.core.index_hierarchy import IndexHierarchy #pylint: disable=W0611 #pragma: no cover
     from static_frame.core.index_auto import IndexAutoFactoryType #pylint: disable=W0611 #pragma: no cover
 
-from static_frame.core.util import IndexConstructor
-from static_frame.core.util import IndexConstructors
-from static_frame.core.util import IndexInitializer
-from static_frame.core.util import STATIC_ATTR
-from static_frame.core.util import AnyCallable
-from static_frame.core.util import NULL_SLICE
-from static_frame.core.util import Bloc2DKeyType
-from static_frame.core.util import DtypesSpecifier
-from static_frame.core.util import slice_to_ascending_slice
-from static_frame.core.util import GetItemKeyType
-from static_frame.core.util import DEFAULT_SORT_KIND
-from static_frame.core.util import iterable_to_array_1d
-
-from static_frame.core.util import DTYPE_OBJECT
-from static_frame.core.util import DTYPE_STR
-from static_frame.core.util import DTYPE_BOOL
-
-from static_frame.core.index_base import IndexBase
-
-
 
 def dtypes_mappable(dtypes: DtypesSpecifier) -> bool:
     '''
@@ -118,12 +125,8 @@ def pandas_to_numpy(
             # if an extension type and it hasna, have to go to object; otherwise, set to None or the dtype obtained above
             dtype = DTYPE_OBJECT if hasna else dtype
 
-        try:
-            array = container.to_numpy(copy=not own_data, dtype=dtype)
-        except (ValueError, TypeError):
-            # cannot convert to '<class 'int'>'-dtype NumPy array with missing values. Specify an appropriate 'na_value' for this dtype; this will go to object
-            # TypeError: boolean value of NA is ambiguous
-            array = container.to_numpy(copy=not own_data)
+        # NOTE: in some cases passing the dtype might raise an exception, but it appears we are handling all of those cases by looking at hasna and selecting an object dtype
+        array = container.to_numpy(copy=not own_data, dtype=dtype)
 
         if hasna:
             # if hasna and extension dtype, should be an object array; please pd.NA objects with fill_value (np.nan)
@@ -141,11 +144,6 @@ def pandas_to_numpy(
 
 
 
-
-
-
-
-
 def index_from_optional_constructor(
         value: IndexInitializer,
         *,
@@ -241,7 +239,7 @@ def matmul(
             aligned = lhs._index.union(rhs._index)
             # if the aligned shape is not the same size as the originals, we do not have the same values in each and cannot proceed (all values go to NaN)
             if len(aligned) != len(lhs._index) or len(aligned) != len(rhs._index):
-                raise RuntimeError('shapes not alignable for matrix multiplication')
+                raise RuntimeError('shapes not alignable for matrix multiplication') #pragma: no cover
 
         if lhs_type == Series:
             if rhs_type == np.ndarray:
@@ -254,13 +252,10 @@ def matmul(
                     index = None # force auto increment integer
                     own_index = False
                     constructor = lhs.__class__
-                # else:
-                #     index = lhs.index
             elif rhs_type == Series:
                 ndim = 0
                 left = lhs.reindex(aligned).values
                 right = rhs.reindex(aligned).values
-                # index = aligned
             else: # rhs is Frame
                 ndim = 1
                 left = lhs.reindex(aligned).values
@@ -330,6 +325,8 @@ def matmul(
                 own_index = False
                 columns = rhs._columns
                 constructor = rhs.__class__
+    else:
+        raise NotImplementedError(f'no handling for {lhs}')
 
     # NOTE: np.matmul is not the same as np.dot for some arguments
     data = np.matmul(left, right)
@@ -530,16 +527,18 @@ def key_to_ascending_key(key: GetItemKeyType, size: int) -> GetItemKeyType:
     raise RuntimeError(f'unhandled key {key}')
 
 
-
-def rehierarch_and_map(*,
-        labels: np.ndarray,
+def rehierarch_from_type_blocks(*,
+        labels: 'TypeBlocks',
         depth_map: tp.Sequence[int],
-        index_constructor: IndexConstructor,
+        index_cls: tp.Type['IndexHierarchy'],
         index_constructors: tp.Optional[IndexConstructors] = None,
         name: tp.Optional[tp.Hashable] = None,
         ) -> tp.Tuple['IndexBase', np.ndarray]:
     '''
     Given labels suitable for a hierarchical index, order them into a hierarchy using the given depth_map.
+
+    Args:
+        index_cls: provide a class, form which the constructor will be called.
     '''
 
     depth = labels.shape[1] # number of columns
@@ -549,32 +548,52 @@ def rehierarch_and_map(*,
     if set(range(depth)) != set(depth_map):
         raise RuntimeError('all depths must be specified')
 
-    labels_post = labels[NULL_SLICE, list(depth_map)]
+    labels_post = labels._extract(row_key=NULL_SLICE, column_key=list(depth_map))
     labels_sort = np.full(labels_post.shape, 0)
 
-    # get ordering of vlues found in each level
+    # get ordering of values found in each level
     order: tp.List[tp.Dict[tp.Hashable, int]] = [defaultdict(int) for _ in range(depth)]
 
-    for idx_row, label in enumerate(labels):
-        label = tuple(label)
-        for idx_col in range(depth):
-            if label[idx_col] not in order[idx_col]:
-                # Map label to an integer representing the observed order.
-                order[idx_col][label[idx_col]] = len(order[idx_col])
-            # Fill array for sorting based on observed order.
-            labels_sort[idx_row, idx_col] = order[idx_col][label[idx_col]]
+    for (idx_row, idx_col), label in labels.element_items():
+        if label not in order[idx_col]:
+            # Map label to an integer representing the observed order.
+            order[idx_col][label] = len(order[idx_col])
+        # Fill array for sorting based on observed order.
+        labels_sort[idx_row, idx_col] = order[idx_col][label]
 
     # Reverse depth_map for lexical sorting, which sorts by rightmost column first.
-    order_lex = np.lexsort([labels_sort[NULL_SLICE, i] for i in reversed(depth_map)])
-    labels_post = labels_post[order_lex]
-    labels_post.flags.writeable = False
-    index = index_constructor(labels_post,
+    order_lex = np.lexsort(
+            [labels_sort[NULL_SLICE, i] for i in reversed(depth_map)])
+
+    labels_post = labels_post._extract(row_key=order_lex)
+
+    index = index_cls._from_type_blocks(
+            blocks=labels_post,
             index_constructors=index_constructors,
             name=name,
+            own_blocks=True,
             )
     return index, order_lex
 
+def rehierarch_from_index_hierarchy(*,
+        labels: 'IndexHierarchy',
+        depth_map: tp.Sequence[int],
+        index_constructors: tp.Optional[IndexConstructors] = None,
+        name: tp.Optional[tp.Hashable] = None,
+        ) -> tp.Tuple['IndexBase', np.ndarray]:
+    '''
+    Alternate interface that updates IndexHierarchy cache before rehierarch.
+    '''
+    if labels._recache:
+        labels._update_array_cache()
 
+    return rehierarch_from_type_blocks(
+            labels=labels._blocks,
+            depth_map=depth_map,
+            index_cls=labels.__class__,
+            index_constructors=index_constructors,
+            name=name,
+            )
 
 def array_from_value_iter(
         key: tp.Hashable,
@@ -622,4 +641,112 @@ def array_from_value_iter(
     return values
 
 
+def apply_binary_operator(*,
+        values: np.ndarray,
+        other: tp.Any,
+        other_is_array: bool,
+        operator: UFunc,
+        ) -> np.ndarray:
+    '''
+    Utility to handle binary operator application.
+    '''
 
+    if (values.dtype.kind in DTYPE_STR_KIND or
+            (other_is_array and other.dtype.kind in DTYPE_STR_KIND)):
+        operator_name = operator.__name__
+
+        if operator_name == 'add':
+            result = npc.add(values, other)
+        elif operator_name == 'radd':
+            result = npc.add(other, values)
+        elif operator_name == 'mul' or operator_name == 'rmul':
+            result = npc.multiply(values, other)
+        else:
+            result = operator(values, other)
+    else:
+        result = operator(values, other)
+
+    if result is False or result is True:
+        # in comparison to Booleans, if values is of length 1 and a character type, we will get a Boolean back, not an array; this issues the following warning: FutureWarning: elementwise comparison failed; returning scalar instead, but in the future will perform elementwise comparison
+        # if the arguement is a tuple of length equal to an erray, NP will perform element wise comparison; but if the argment is a tuple of length greater or equal, each value in value will be compared to that tuple
+        result = np.full(values.shape, result)
+
+    result.flags.writeable = False
+    return result
+
+
+def apply_binary_operator_blocks(*,
+        values: tp.Iterable[np.ndarray],
+        other: tp.Iterable[np.ndarray],
+        operator: UFunc,
+        apply_column_2d_filter: bool,
+    ) -> tp.Iterator[np.ndarray]:
+    '''
+    Application from iterators of arrays, to iterators of arrays.
+    '''
+
+    if apply_column_2d_filter:
+        values = (column_2d_filter(op) for op in values)
+        other = (column_2d_filter(op) for op in other)
+
+    for a, b in zip_longest(values, other):
+        yield apply_binary_operator(
+                values=a,
+                other=b,
+                other_is_array=True,
+                operator=operator,
+                )
+
+
+def arrays_from_index_frame(
+        container: 'Frame',
+        depth_level: tp.Optional[DepthLevelSpecifier],
+        columns: GetItemKeyType
+        ) -> tp.Iterator[np.ndarray]:
+    '''
+    Given a Frame, return an iterator of index and / or columns as 1D or 2D arrays.
+    '''
+    if depth_level is not None:
+        # NOTE: a 1D index of tuples will be taken as a 1D array of tuples; there is no obvious way to treat this as 2D array without guessing that we are trying to match an IndexHierarchy
+        # NOTE: if a multi-column selection, might be better to yield one depth at a time
+        yield container.index.values_at_depth(depth_level)
+    if columns is not None:
+        column_key = container.columns.loc_to_iloc(columns)
+        yield from container._blocks._slice_blocks(column_key=column_key)
+
+
+def key_from_container_key(
+        index: IndexBase,
+        key: GetItemKeyType,
+        expand_iloc: bool = False,
+        ) -> GetItemKeyType:
+
+    from static_frame.core.index import Index
+    from static_frame.core.index import ILoc
+    from static_frame.core.series import Series
+
+    if isinstance(key, Index):
+        # if an Index, we simply use the values of the index
+        key = key.values
+    elif isinstance(key, Series):
+        if key.dtype == bool:
+            # if a Boolean series, sort and reindex
+            if not key.index.equals(index):
+                key = key.reindex(index,
+                        fill_value=False,
+                        check_equals=False,
+                        ).values
+            else: # the index is equal
+                key = key.values
+        else:
+            # For all other Series types, we simply assume that the values are to be used as keys in the IH. This ignores the index, but it does not seem useful to require the Series, used like this, to have a matching index value, as the index and values would need to be identical to have the desired selection.
+            key = key.values
+    elif expand_iloc and isinstance(key, ILoc):
+        # realize as Boolean array
+        array = np.full(len(index), False)
+        array[key.key] = True
+        key = array
+
+    # detect and fail on Frame?
+
+    return key
\ No newline at end of file
