@@ -16,14 +16,13 @@ from collections import namedtuple
 import numpy as np
 
 
-from static_frame.core.util import _gen_skip_middle
-from static_frame.core.display_color import HexColor
 from static_frame.core import display_html_datatables
-
+from static_frame.core.display_color import HexColor
+from static_frame.core.util import _gen_skip_middle
+from static_frame.core.util import COMPLEX_TYPES
 from static_frame.core.util import DTYPE_INT_KIND
 from static_frame.core.util import DTYPE_STR_KIND
 from static_frame.core.util import FLOAT_TYPES
-from static_frame.core.util import COMPLEX_TYPES
 
 _module = sys.modules[__name__]
 
@@ -333,7 +332,6 @@ class DisplayFormatRST(DisplayFormat):
         # body lines add boundary lines below
         def lines() -> tp.Iterator[str]:
             for line in msg.split(cls.LINE_SEP):
-                # import ipdb; ipdb.set_trace()
                 yield line
                 yield cls._RE_NOT_PIPE.sub('-', line).replace('|', '+')
 
@@ -482,6 +480,9 @@ class DisplayConfig:
             'display_columns',
             'display_rows',
 
+            'include_columns',
+            'include_index',
+
             'cell_max_width',
             'cell_max_width_leftmost',
             'cell_align_left',
@@ -538,9 +539,11 @@ class DisplayConfig:
             display_columns: int = 12,
             display_rows: int = 36,
 
+            include_columns: bool = True,
+            include_index: bool = True,
+
             cell_max_width: int = 20,
             cell_max_width_leftmost: int = 36,
-
             cell_align_left: bool = True
             ) -> None:
 
@@ -570,15 +573,24 @@ class DisplayConfig:
         self.value_format_complex_scientific = value_format_complex_scientific
 
         self.display_format = display_format
-
         self.display_columns = display_columns
         self.display_rows = display_rows
 
+        self.include_columns = include_columns
+        self.include_index = include_index
+
         self.cell_max_width = cell_max_width
         self.cell_max_width_leftmost = cell_max_width_leftmost
-
         self.cell_align_left = cell_align_left
 
+        #-----------------------------------------------------------------------
+        # handle any inter-dependent configurations
+
+        if not self.include_columns or not self.include_index:
+            if self.type_show:
+                raise RuntimeError('cannot show types if not including columns or index.')
+
+
     def write(self, fp: str) -> None:
         '''Write a JSON file.
         '''
@@ -700,8 +712,6 @@ class DisplayActive:
 
 
 #-------------------------------------------------------------------------------
-
-
 class DisplayHeader:
     '''
     Wraper for passing in display header that have a name attribute.
@@ -815,7 +825,7 @@ class Display:
             config: DisplayConfig,
             is_dtype: bool = False) -> DisplayCell:
         '''
-        Given a raw value, retrun a DisplayCell, which is defined as a pair of the string representation and the character width without any formatting markup.
+        Given a raw value, return a :obj:`DisplayCell`.
         '''
         if is_dtype or inspect.isclass(value) or isinstance(value, DisplayHeader):
             type_str_raw, type_category = cls.type_attributes(
@@ -874,6 +884,7 @@ class Display:
         # create a list of lists, always starting with the header
         rows = []
         if header is not None:
+            # NOTE: controlling if the header is applied with type_show is moving to display() methods; this approach will no longer be needed
             # assume that all headers are SF types; skip if type_show is False
             if config.type_show:
                 rows.append([cls.to_cell(header, config=config)])
@@ -1071,24 +1082,6 @@ class Display:
 
         return rows
 
-        # post = []
-        # for row_idx, row in enumerate(rows):
-        #     # make sure the entire row is marked as head if row index less than column depth
-        #     added_depth = col_count_src if row_idx < columns_depth else 0
-                # rstrip to remove extra white space on last column
-        #     post.append(''.join(dfc.markup_row(row,
-        #             header_depth=index_depth + added_depth)).rstrip()
-        #             )
-        # return post
-
-    @staticmethod
-    def _row_empty(line: tp.Iterable[str]) -> bool:
-        for cell in line:
-            if cell.strip() != '':
-                return False
-        return True
-
-
     #---------------------------------------------------------------------------
     def __init__(self,
             rows: tp.List[tp.List[DisplayCell]],
@@ -1113,8 +1106,6 @@ class Display:
     def __repr__(self) -> str:
         rows = self._to_rows_cells(self,
                 self._config,
-                # index_depth=self._index_depth,
-                # header_depth=self._header_depth
                 )
 
         if self._outermost:
@@ -1123,18 +1114,24 @@ class Display:
             body = []
             for idx, row in enumerate(rows):
                 if idx < self._header_depth:
-                    # if we find an empty header, it is due to setting type_show to False; we can skip them here
-                    if self._row_empty(row):
-                        continue
-                    row = ''.join(dfc.markup_row(row, header_depth=np.inf)).rstrip()
+                    row = ''.join(dfc.markup_row(row,
+                            header_depth=np.inf
+                            )).rstrip()
                     header.append(row)
                 else:
-                    row = ''.join(dfc.markup_row(row, header_depth=self._index_depth)).rstrip()
+                    row = ''.join(dfc.markup_row(row,
+                            header_depth=self._index_depth
+                            )).rstrip()
                     body.append(row)
-            header_str = dfc.markup_header(dfc.LINE_SEP.join(header))
+
+            outermost = []
+            if header:
+                header_str = dfc.markup_header(dfc.LINE_SEP.join(header))
+                outermost.append(header_str)
+
             body_str = dfc.markup_body(dfc.LINE_SEP.join(body))
-            # NOTE: this function might take additional arguments (like identifier and caption); presently there is now way to get these args here via container display methods; likely best option is to take a new / specialzied confiig. Using the same config would be akward, as that config is for all outputs, not just one table's output.
-            return dfc.markup_outermost(header_str + dfc.LINE_SEP + body_str)
+            outermost.append(body_str)
+            return dfc.markup_outermost(dfc.LINE_SEP.join(outermost))
 
         return dfc.LINE_SEP.join(''.join(r) for r in rows)
 
@@ -1238,15 +1235,16 @@ class Display:
         rows.extend(self._rows[insert_index:])
         self._rows = rows
 
-    def append_row(self, row: tp.List[DisplayCell]) -> None:
-        '''
-        Append a row at the bottom of existing rows.
-        '''
-        self._rows.append(row)
+    # def drop_row(self, index: int = 0) -> None:
+    #     '''Remove a row in place.
+    #     '''
+    #     self._rows = self._rows[:index] + self._rows[index+1:]
+
 
     #---------------------------------------------------------------------------
     # return a new display
 
+
     def flatten(self) -> 'Display':
         '''
         Return a Display from this Display that is a single row, formed the contents of all rows put in a single senquece, from the top down. Through this a single-column index Display can be made into a single row Display.
@@ -1269,3 +1267,5 @@ class Display:
             for idx, cell in enumerate(r):
                 rows[idx].append(cell)
         return self.__class__(rows, config=self._config)
+
+
