@@ -9,73 +9,70 @@ import numpy as np
 from automap import AutoMap
 from automap import FrozenAutoMap
 
-from static_frame.core.util import DEFAULT_SORT_KIND
-from static_frame.core.util import NULL_SLICE
-from static_frame.core.util import EMPTY_TUPLE
-from static_frame.core.util import EMPTY_SLICE
-from static_frame.core.util import SLICE_ATTRS
-from static_frame.core.util import SLICE_START_ATTR
-from static_frame.core.util import SLICE_STOP_ATTR
-from static_frame.core.util import BOOL_TYPES
-from static_frame.core.util import KEY_ITERABLE_TYPES
-from static_frame.core.util import EMPTY_ARRAY
-from static_frame.core.util import DTYPE_DATETIME_KIND
 
-from static_frame.core.util import GetItemKeyType
-from static_frame.core.util import KeyTransformType
-from static_frame.core.util import CallableOrMapping
-from static_frame.core.util import DtypeSpecifier
-from static_frame.core.util import KeyIterableTypes
-from static_frame.core.util import UFunc
-from static_frame.core.util import NameType
-
-from static_frame.core.util import IndexInitializer
-from static_frame.core.util import DepthLevelSpecifier
-from static_frame.core.util import ufunc_axis_skipna
-from static_frame.core.util import iterable_to_array_1d
-from static_frame.core.util import isin
-
-from static_frame.core.util import immutable_filter
-from static_frame.core.util import name_filter
-from static_frame.core.util import array_shift
-from static_frame.core.util import array2d_to_tuples
-from static_frame.core.util import slice_to_inclusive_slice
-
-from static_frame.core.util import DTYPE_INT_DEFAULT
-
-from static_frame.core.selector_node import InterfaceGetItem
-from static_frame.core.selector_node import InterfaceSelectDuo
-from static_frame.core.selector_node import TContainer
-
-from static_frame.core.util import union1d
-from static_frame.core.util import intersect1d
-from static_frame.core.util import setdiff1d
-from static_frame.core.util import to_datetime64
-from static_frame.core.util import INT_TYPES
-from static_frame.core.util import mloc
-
-from static_frame.core.util import resolve_dtype
 from static_frame.core.container import ContainerOperand
+from static_frame.core.container_util import apply_binary_operator
 from static_frame.core.container_util import matmul
+from static_frame.core.container_util import key_from_container_key
 
-
-from static_frame.core.doc_str import doc_inject
-from static_frame.core.index_base import IndexBase
-# from static_frame.core.iter_node import IterNode
-from static_frame.core.iter_node import IterNodeDepthLevel
-from static_frame.core.iter_node import IterNodeType
-from static_frame.core.iter_node import IterNodeApplyType
-
-
-from static_frame.core.display import DisplayConfig
-from static_frame.core.display import DisplayActive
 from static_frame.core.display import Display
+from static_frame.core.display import DisplayActive
+from static_frame.core.display import DisplayConfig
 from static_frame.core.display import DisplayHeader
-
+from static_frame.core.doc_str import doc_inject
 from static_frame.core.exception import ErrorInitIndex
 from static_frame.core.exception import LocEmpty
 from static_frame.core.exception import LocInvalid
+from static_frame.core.index_base import IndexBase
+from static_frame.core.node_dt import InterfaceDatetime
+from static_frame.core.node_iter import IterNodeApplyType
+from static_frame.core.node_iter import IterNodeDepthLevel
+from static_frame.core.node_iter import IterNodeType
+from static_frame.core.node_selector import InterfaceGetItem
+from static_frame.core.node_selector import InterfaceSelectDuo
+from static_frame.core.node_selector import TContainer
+from static_frame.core.node_str import InterfaceString
 
+from static_frame.core.util import array_shift
+from static_frame.core.util import array2d_to_tuples
+from static_frame.core.util import BOOL_TYPES
+from static_frame.core.util import CallableOrMapping
+from static_frame.core.util import DEFAULT_SORT_KIND
+from static_frame.core.util import DepthLevelSpecifier
+from static_frame.core.util import DTYPE_DATETIME_KIND
+from static_frame.core.util import DTYPE_INT_DEFAULT
+from static_frame.core.util import DTYPE_BOOL
+from static_frame.core.util import DtypeSpecifier
+from static_frame.core.util import EMPTY_ARRAY
+from static_frame.core.util import EMPTY_SLICE
+from static_frame.core.util import EMPTY_TUPLE
+from static_frame.core.util import GetItemKeyType
+from static_frame.core.util import immutable_filter
+from static_frame.core.util import IndexInitializer
+from static_frame.core.util import INT_TYPES
+from static_frame.core.util import intersect1d
+from static_frame.core.util import isin
+from static_frame.core.util import isna_array
+from static_frame.core.util import iterable_to_array_1d
+from static_frame.core.util import KEY_ITERABLE_TYPES
+from static_frame.core.util import KeyIterableTypes
+from static_frame.core.util import KeyTransformType
+from static_frame.core.util import mloc
+from static_frame.core.util import NAME_DEFAULT
+from static_frame.core.util import name_filter
+from static_frame.core.util import NameType
+from static_frame.core.util import NULL_SLICE
+from static_frame.core.util import resolve_dtype
+from static_frame.core.util import setdiff1d
+from static_frame.core.util import SLICE_ATTRS
+from static_frame.core.util import SLICE_START_ATTR
+from static_frame.core.util import SLICE_STEP_ATTR
+from static_frame.core.util import SLICE_STOP_ATTR
+from static_frame.core.util import slice_to_inclusive_slice
+from static_frame.core.util import to_datetime64
+from static_frame.core.util import UFunc
+from static_frame.core.util import ufunc_axis_skipna
+from static_frame.core.util import union1d
 
 if tp.TYPE_CHECKING:
     import pandas #pylint: disable=W0611 #pragma: no cover
@@ -124,13 +121,17 @@ class LocMap:
             attr = getattr(key, field)
             if attr is None:
                 yield None
+
             elif isinstance(attr, np.datetime64):
                 # if a datetime, we assume that the labels are ordered;
                 if attr.dtype == labels.dtype: #type: ignore
-                    pos: int = label_to_pos(attr)
-                    if pos is None:
-                        # if same type, and that atter is not in labels, we fail, just as we do in then non-datetime64 case. Only when datetimes are given in a different unit are we "loose" about matching.
-                        raise LocInvalid('Invalid loc given in a slice', attr, field)
+                    if field != SLICE_STEP_ATTR:
+                        pos: int = label_to_pos(attr)
+                        if pos is None:
+                            # if same type, and that atter is not in labels, we fail, just as we do in then non-datetime64 case. Only when datetimes are given in a different unit are we "loose" about matching.
+                            raise LocInvalid('Invalid loc given in a slice', attr, field)
+                    else: # step
+                        pos = attr # should be an integer
 
                     if field == SLICE_STOP_ATTR:
                         pos += 1 # stop is inclusive
@@ -155,25 +156,30 @@ class LocMap:
                     else:
                         raise LocEmpty()
 
-                if offset_apply:
+                elif field == SLICE_STEP_ATTR:
+                    pos = attr
+
+                if offset_apply and field != SLICE_STEP_ATTR:
                     pos += offset #type: ignore
 
                 yield pos
 
             else:
-                pos = label_to_pos(attr)
-                if pos is None:
-                    # NOTE: could raise LocEmpty() to silently handle this
-                    raise LocInvalid('Invalid loc given in a slice', attr, field)
+                if field != SLICE_STEP_ATTR:
+                    pos = label_to_pos(attr)
+                    if pos is None:
+                        # NOTE: could raise LocEmpty() to silently handle this
+                        raise LocInvalid('Invalid loc given in a slice', attr, field)
+                    if offset_apply:
+                        pos += offset #type: ignore
+                else: # step
+                    pos = attr # should be an integer
 
-                if offset_apply:
-                    pos += offset
-
-                if field is SLICE_STOP_ATTR:
+                if field == SLICE_STOP_ATTR:
                     # loc selections are inclusive, so iloc gets one more
-                    yield pos + 1
-                else:
-                    yield pos
+                    pos += 1
+
+                yield pos
 
     @classmethod
     def loc_to_iloc(cls, *,
@@ -181,13 +187,15 @@ class LocMap:
             labels: np.ndarray,
             positions: np.ndarray,
             key: GetItemKeyType,
-            offset: tp.Optional[int] = None
+            offset: tp.Optional[int] = None,
+            partial_selection: bool = False,
             ) -> GetItemKeyType:
         '''
         Note: all SF objects (Series, Index) need to be converted to basic types before being passed as `key` to this function.
 
         Args:
             offset: in the contect of an IndexHierarchical, the iloc positions returned from this funcition need to be shifted.
+            partial_selection: if True and key is an iterable of labels that includes lables not in the mapping, available matches will be returned rather than raising.
         Returns:
             An integer mapped slice, or GetItemKey type that is based on integers, compatible with TypeBlocks
         '''
@@ -200,8 +208,8 @@ class LocMap:
                 # when offset is defined (even if it is zero), null slice is not sufficiently specific; need to convert to an explict slice relative to the offset
                 return slice(offset, len(positions) + offset) #type: ignore
             try:
-                return slice(*cls.map_slice_args( #type: ignore
-                        label_to_pos.get,
+                return slice(*cls.map_slice_args(
+                        label_to_pos.get, #type: ignore
                         key,
                         labels,
                         offset)
@@ -215,7 +223,6 @@ class LocMap:
                 key = labels.astype(key.dtype) == key
             # if not different type, keep it the same so as to do a direct, single element selection
 
-        # handles only lists and arrays; break out comparisons to avoid multiple
         is_array = isinstance(key, np.ndarray)
         is_list = isinstance(key, list)
 
@@ -224,22 +231,24 @@ class LocMap:
             if is_array and key.dtype.kind == DTYPE_DATETIME_KIND:
                 if labels.dtype != key.dtype:
                     labels_ref = labels.astype(key.dtype)
-                    # let Boolean key hit next branch
-                    key = reduce(operator_mod.or_,
-                            (labels_ref == k for k in key))
-                    # NOTE: may want to raise instead of support this
-                    # raise NotImplementedError(f'selecting {labels.dtype} with {key.dtype} is not presently supported')
+                    # let Boolean key advance to next branch
+                    key = reduce(operator_mod.or_, (labels_ref == k for k in key))
 
-            if is_array and key.dtype == bool:
+            if is_array and key.dtype is DTYPE_BOOL:
                 if offset_apply:
                     return positions[key] + offset
                 return positions[key]
 
-            # map labels to integer positions
-            # NOTE: we may miss the opportunity to get a reference from values when we have contiguous keys
+            # map labels to integer positions, return a list of integer positions
+            # NOTE: we may miss the opportunity to identify contiguous keys and extract a slice
+            # NOTE: we do more branching here to optimize performance
+            if partial_selection:
+                if offset_apply:
+                    return [label_to_pos[k] + offset for k in key if k in label_to_pos] #type: ignore
+                return [label_to_pos[k] for k in key if k in label_to_pos]
             if offset_apply:
-                return [label_to_pos[x] + offset for x in key] #type: ignore
-            return [label_to_pos[x] for x in key]
+                return [label_to_pos[k] + offset for k in key] #type: ignore
+            return [label_to_pos[k] for k in key]
 
         # if a single element (an integer, string, or date, we just get the integer out of the map
         if offset_apply:
@@ -387,7 +396,7 @@ class Index(IndexBase):
             labels: IndexInitializer,
             *,
             loc_is_iloc: bool = False,
-            name: NameType = None,
+            name: NameType = NAME_DEFAULT,
             dtype: DtypeSpecifier = None
             ) -> None:
 
@@ -412,7 +421,7 @@ class Index(IndexBase):
         if isinstance(labels, IndexBase):
             if labels._recache:
                 labels._update_array_cache()
-            if name is None and labels.name is not None:
+            if name is NAME_DEFAULT:
                 name = labels.name # immutable, so no copy necessary
             if isinstance(labels, Index): # not an IndexHierarchy
                 if (labels.STATIC and self.STATIC and dtype is None):
@@ -446,7 +455,7 @@ class Index(IndexBase):
                 labels = labels.astype(dtype_extract)
                 labels.flags.writeable = False #type: ignore
 
-        self._name = name if name is None else name_filter(name)
+        self._name = None if name is NAME_DEFAULT else name_filter(name)
 
         if self._map is None: # if _map not shared from another Index
             if not loc_is_iloc:
@@ -486,7 +495,7 @@ class Index(IndexBase):
     #---------------------------------------------------------------------------
     # name interface
 
-    def rename(self: I, name: tp.Hashable) -> I:
+    def rename(self: I, name: NameType) -> I:
         '''
         Return a new Frame with an updated name attribute.
         '''
@@ -662,6 +671,47 @@ class Index(IndexBase):
                 name=self._name
                 )
 
+
+    #---------------------------------------------------------------------------
+    @property
+    def via_str(self) -> InterfaceString[np.ndarray]:
+        '''
+        Interface for applying string methods to elements in this container.
+        '''
+        if self._recache:
+            self._update_array_cache()
+
+        blocks = (self._labels,)
+        cls = Index if self.STATIC else IndexGO
+
+        def blocks_to_container(blocks: tp.Iterator[np.ndarray]) -> np.ndarray:
+            return next(blocks)
+
+        return InterfaceString(
+                blocks=blocks,
+                blocks_to_container=blocks_to_container,
+                )
+
+    @property
+    def via_dt(self) -> InterfaceDatetime[np.ndarray]:
+        '''
+        Interface for applying datetime properties and methods to elements in this container.
+        '''
+        if self._recache:
+            self._update_array_cache()
+
+        blocks = (self.values,)
+        cls = Index if self.STATIC else IndexGO
+
+        def blocks_to_container(blocks: tp.Iterator[np.ndarray]) -> np.ndarray:
+            return next(blocks)
+
+        return InterfaceDatetime(
+                blocks=blocks,
+                blocks_to_container=blocks_to_container,
+                )
+
+
     #---------------------------------------------------------------------------
 
     def _update_array_cache(self) -> None:
@@ -689,12 +739,21 @@ class Index(IndexBase):
         if self._recache:
             self._update_array_cache()
 
+        header: tp.Optional[DisplayHeader]
+
+        if config.type_show:
+            header = DisplayHeader(self.__class__, self._name)
+            header_depth = 1
+        else:
+            header = None
+            header_depth = 0
+
         return Display.from_values(self.values,
-                header=DisplayHeader(self.__class__, self._name),
+                header=header,
                 config=config,
                 outermost=True,
                 index_depth=0,
-                header_depth=1
+                header_depth=header_depth,
                 )
 
     #---------------------------------------------------------------------------
@@ -776,7 +835,8 @@ class Index(IndexBase):
     def loc_to_iloc(self,
             key: GetItemKeyType,
             offset: tp.Optional[int] = None,
-            key_transform: KeyTransformType = None
+            key_transform: KeyTransformType = None,
+            partial_selection: bool = False,
             ) -> GetItemKeyType:
         '''
         Note: Boolean Series are reindexed to this index, then passed on as all Boolean arrays.
@@ -794,17 +854,8 @@ class Index(IndexBase):
 
         if isinstance(key, ILoc):
             return key.key
-        elif isinstance(key, Index):
-            # if an Index, we simply use the values of the index
-            key = key.values
-        elif isinstance(key, Series):
-            if key.dtype == bool:
-                if _requires_reindex(key.index, self):
-                    key = key.reindex(self, fill_value=False).values
-                else: # the index is equal
-                    key = key.values
-            else:
-                key = key.values
+
+        key = key_from_container_key(self, key)
 
         if self._map is None: # loc_is_iloc
             if isinstance(key, np.ndarray):
@@ -826,7 +877,8 @@ class Index(IndexBase):
                 labels=self._labels,
                 positions=self._positions, # always an np.ndarray
                 key=key,
-                offset=offset
+                offset=offset,
+                partial_selection=partial_selection,
                 )
 
     def _extract_iloc(self,
@@ -891,26 +943,26 @@ class Index(IndexBase):
         if self._recache:
             self._update_array_cache()
 
+        values = self._labels
+        other_is_array = False
+
         if issubclass(other.__class__, Index):
             other = other.values # operate on labels to labels
+            other_is_array = True
+        elif isinstance(other, np.ndarray):
+            other_is_array = True
 
         if operator.__name__ == 'matmul':
-            return matmul(self._labels, other)
+            return matmul(values, other)
         elif operator.__name__ == 'rmatmul':
-            return matmul(other, self._labels)
-
-        result = operator(self._labels, other)
-
-        if not isinstance(result, np.ndarray):
-            # see Series._ufunc_binary_operator for notes on why
-            if isinstance(result, BOOL_TYPES):
-                result = np.full(len(self._labels), result)
-            else:
-                raise RuntimeError('unexpected branch from non-array result of operator application to array') #pragma: no cover
-
-        result.flags.writeable = False
-        return result
+            return matmul(other, values)
 
+        return apply_binary_operator(
+                values=values,
+                other=other,
+                other_is_array=other_is_array,
+                operator=operator,
+                )
 
     def _ufunc_axis_skipna(self, *,
             axis: int,
@@ -974,6 +1026,60 @@ class Index(IndexBase):
     #---------------------------------------------------------------------------
     # utility functions
 
+    @doc_inject()
+    def equals(self,
+            other: tp.Any,
+            *,
+            compare_name: bool = False,
+            compare_dtype: bool = False,
+            compare_class: bool = False,
+            skipna: bool = True,
+            ) -> bool:
+        '''
+        {doc}
+
+        Args:
+            {compare_name}
+            {compare_dtype}
+            {compare_class}
+            {skipna}
+        '''
+
+        if id(other) == id(self):
+            return True
+
+        if compare_class and self.__class__ != other.__class__:
+            return False
+        elif not isinstance(other, Index):
+            return False
+
+        # defer updating cache
+        if self._recache:
+            self._update_array_cache()
+
+        # same type from here
+        if len(self) != len(other):
+            return False
+        if compare_name and self.name != other.name:
+            return False
+        if compare_dtype and self.dtype != other.dtype:
+            return False
+
+        eq = self.values == other.values
+
+        # NOTE: will only be False, or an array
+        if eq is False:
+            return eq #type: ignore
+
+        if skipna:
+            isna_both = (isna_array(self.values, include_none=False)
+                    & isna_array(other.values, include_none=False))
+            eq[isna_both] = True
+
+        if not eq.all():
+            return False
+        return True
+
     def sort(self,
             ascending: bool = True,
             kind: str = DEFAULT_SORT_KIND) -> 'Index':
@@ -1015,9 +1121,9 @@ class Index(IndexBase):
     def to_series(self) -> 'Series':
         '''Return a Series with values from this Index's labels.
         '''
-        # not sure if index should be self here
+        # NOTE: while we might re-use the index on the index returned from this Series, such an approach will not work with IndexHierarchy.to_frame, as we do not know if the index should be on the index or columns; thus, returning an unindexed Series is appropriate
         from static_frame import Series
-        return Series(self.values, index=None, name=self._name)
+        return Series(self.values, name=self._name)
 
     def add_level(self, level: tp.Hashable) -> 'IndexHierarchy':
         '''Return an IndexHierarhcy with an added root level.
@@ -1170,21 +1276,3 @@ def _index_initializer_needs_init(
         return bool(len(value))
     return bool(value)
 
-
-def _requires_reindex(left: Index, right: Index) -> bool:
-    '''
-    Given two Index objects, determine if we need to reindex
-    '''
-    if len(left) != len(right):
-        return True
-    # do not need a new Index object, so just compare arrays directly, which might return a single Boolean if the types are not compatible
-
-    # NOTE: NP raises a warning here if we go to scalar value
-    ne = left.values != right.values
-    if isinstance(ne, np.ndarray):
-        # if any not equal, require reindex
-        return ne.any() #type: ignore
-    # assume we have a bool
-    return ne # if not equal, require reindex
-
-
