@@ -6,25 +6,30 @@ from itertools import zip_longest
 
 import numpy as np
 
-from static_frame.core.hloc import HLoc
-from static_frame.core.index import Index
-from static_frame.core.index import ILoc
-from static_frame.core.index import IndexGO
+
 from static_frame.core.array_go import ArrayGO
-
-from static_frame.core.util import KEY_MULTIPLE_TYPES
-from static_frame.core.util import KEY_ITERABLE_TYPES
-from static_frame.core.util import INT_TYPES
-from static_frame.core.util import GetItemKeyType
-
-from static_frame.core.util import resolve_dtype_iter
-from static_frame.core.util import GetItemKeyTypeCompound
-
-from static_frame.core.type_blocks import TypeBlocks
-
+from static_frame.core.container_util import index_from_optional_constructor
+from static_frame.core.doc_str import doc_inject
+from static_frame.core.exception import ErrorInitIndexLevel
+from static_frame.core.hloc import HLoc
+from static_frame.core.index import ILoc
+from static_frame.core.index import Index
+from static_frame.core.index import IndexGO
 from static_frame.core.index import LocMap
 from static_frame.core.index import mutable_immutable_index_filter
-from static_frame.core.exception import ErrorInitIndexLevel
+from static_frame.core.index_base import IndexBase
+from static_frame.core.type_blocks import TypeBlocks
+from static_frame.core.util import GetItemKeyType
+from static_frame.core.util import GetItemKeyTypeCompound
+from static_frame.core.util import IndexConstructor
+from static_frame.core.util import IndexConstructors
+from static_frame.core.util import IndexInitializer
+from static_frame.core.util import INT_TYPES
+from static_frame.core.util import DTYPE_BOOL
+from static_frame.core.util import KEY_ITERABLE_TYPES
+from static_frame.core.util import KEY_MULTIPLE_TYPES
+from static_frame.core.util import resolve_dtype_iter
+
 
 # if tp.TYPE_CHECKING:
 #     from static_frame.core.type_blocks import TypeBlocks #pylint: disable=W0611 #pragma: no cover
@@ -37,6 +42,8 @@ INDEX_LEVEL_SLOTS = (
             '_length',
             )
 
+
+
 class IndexLevel:
     '''
     A nestable representation of an Index, where labels in that index optionally point to other Index objects.
@@ -49,6 +56,70 @@ class IndexLevel:
     _length: tp.Optional[int]
 
     STATIC: bool = True
+    _INDEX_CONSTRUCTOR = Index
+
+    @classmethod
+    def from_level_data(cls,
+            level_data: tp.Any,
+            get_index: tp.Callable[[IndexInitializer, int], IndexBase],
+            offset: int = 0,
+            depth: int = 0,
+            ) -> 'IndexLevel':
+        '''
+        Recursive function used in ``from_tree`` constructor.
+        '''
+        if isinstance(level_data, dict):
+            level_labels = []
+            targets = np.empty(len(level_data), dtype=object)
+            offset_local = 0
+
+            # ordered key, value pairs, where the key is the label, the value is a list or dictionary; enmerate for insertion pre-allocated object array
+            for idx, (k, v) in enumerate(level_data.items()):
+                level_labels.append(k)
+                level = cls.from_level_data(v,
+                        get_index,
+                        offset=offset_local,
+                        depth=depth + 1)
+                targets[idx] = level
+                offset_local += len(level) # for lower level offsetting
+
+            index = get_index(level_labels, depth)
+            targets = ArrayGO(targets, own_iterable=True)
+
+        else: # an iterable, terminal node, no offsets needed
+            index = get_index(level_data, depth)
+            targets = None
+
+        return cls(
+                index=index, #type: ignore
+                offset=offset,
+                targets=targets,
+                )
+
+    @classmethod
+    def from_tree(cls,
+            tree: tp.Any, # recersively defined
+            index_constructors: tp.Optional[IndexConstructors] = None,
+            ) -> 'IndexLevel':
+        '''
+        Convert a tree structure to an IndexLevel instance.
+        '''
+        # tree: tp.Dict[tp.Hashable, tp.Union[Sequence[tp.Hashable], tp.Dict]]
+
+        def get_index(labels: IndexInitializer, depth: int) -> IndexBase:
+            explicit_constructor: tp.Optional[IndexConstructor]
+
+            if index_constructors is not None:
+                explicit_constructor = index_constructors[depth]
+            else:
+                explicit_constructor = None
+
+            return index_from_optional_constructor(labels,
+                    default_constructor=cls._INDEX_CONSTRUCTOR,
+                    explicit_constructor=explicit_constructor)
+
+        return cls.from_level_data(tree, get_index)
+
 
     def __init__(self,
             index: Index,
@@ -100,6 +171,9 @@ class IndexLevel:
         '''
         Assuming all depths are uniform, can get the depth without storing levels list. Could store a depth attribute, but all nested components with provide overlapping depth descriptions that are never examined.
         '''
+        if not len(self.index): # if zero sized, depth is zero
+            # TODO: need a way to represent 0-length IndexLevels with non-zero depth
+            return 1
         if self.targets is None:
             return 1
         level, depth = self, 1
@@ -268,6 +342,9 @@ class IndexLevel:
     def __contains__(self, key: tp.Iterable[tp.Hashable]) -> bool:
         '''Given an iterable of single-element level keys (a leaf loc), return a bool.
         '''
+        if not hasattr(key, '__iter__') or isinstance(key, str):
+            return False
+
         node = self
         for k in key:
             if not node.index.__contains__(k):
@@ -319,41 +396,57 @@ class IndexLevel:
 
         If key is an np.ndarray, a Boolean array will be passed through; otherwise, it will be treated as an iterable of values to be passed to leaf_loc_to_iloc.
         '''
+        from static_frame.core.series import Series
+
         if isinstance(key, slice):
             # given a top-level definition of a slice (and if that slice results in a single value), we can get a value range
             return slice(*LocMap.map_slice_args(self.leaf_loc_to_iloc, key))
 
-        # this should not match tuples that are leaf-locs
-        if isinstance(key, KEY_ITERABLE_TYPES):
+        if isinstance(key, KEY_ITERABLE_TYPES): # iterables of leaf-locs
             if isinstance(key, np.ndarray) and key.dtype == bool:
                 return key # keep as Boolean
             return [self.leaf_loc_to_iloc(x) for x in key]
 
-        if not isinstance(key, HLoc):
-            # assume it is a leaf loc tuple
+        if not isinstance(key, HLoc): # assume a leaf loc tuple
             if not isinstance(key, tuple):
                 raise KeyError(f'{key} cannot be used for loc selection from IndexHierarchy; try HLoc')
             return self.leaf_loc_to_iloc(key)
 
-        # everything after this is an HLoc
-        # collect all ilocs for all leaf indices matching HLoc patterns
+        # HLoc following: collect all ilocs for all leaf indices matching HLoc patterns
         ilocs = []
         levels = deque(((self, 0, 0),)) # order matters
 
         while levels:
             level, depth, offset = levels.popleft()
+
             depth_key = key[depth]
+            # NOTE: depth_key should not be Series or Index at this point; IndexHierarchy is responsible for unpacking / reindexing prior to this call
             next_offset = offset + level.offset
 
+            if isinstance(depth_key, np.ndarray) and depth_key.dtype is DTYPE_BOOL:
+                # NOTE: use length of level, not length of index, as need to observe all leafs covered at this node.
+                depth_key = depth_key[next_offset: next_offset + len(level)]
+                if len(depth_key) > len(level.index):
+                    # given leaf-Boolean, determine what upper nodes to select
+                    depth_key = level.values_at_depth(0)[depth_key]
+                    if len(depth_key) > 1:
+                        # NOTE: must strip repeated labels, but cannot us np.unique as must retain order
+                        depth_key = list(dict.fromkeys(depth_key).keys())
+
             # print(level, depth, offset, depth_key, next_offset)
             if level.targets is None:
                 try:
-                    ilocs.append(level.index.loc_to_iloc(depth_key, offset=next_offset))
+                    # NOTE: as a selection list might be given within the HLoc, it will be tested accross many indices, and should support a partial matching
+                    ilocs.append(level.index.loc_to_iloc(
+                            depth_key,
+                            offset=next_offset,
+                            partial_selection=True,
+                            ))
                 except KeyError:
                     pass
-            else: # target is iterable np.ndaarray
-                try:
-                    iloc = level.index.loc_to_iloc(depth_key) # no offset
+            else: # when not at a leaf, we are selecting level_targets to descend withing
+                try: # NOTE: no offset necessary as not a leaf selection
+                    iloc = level.index.loc_to_iloc(depth_key, partial_selection=True)
                 except KeyError:
                     pass
                 else:
@@ -368,12 +461,10 @@ class IndexLevel:
 
         iloc_count = len(ilocs)
         if iloc_count == 0:
-            # import ipdb; ipdb.set_trace()
             raise KeyError('no matching keys across all levels')
 
         if iloc_count == 1 and not key.has_key_multiple():
-            # drop to a single iloc selection
-            return ilocs[0]
+            return ilocs[0] # drop to a single iloc selection
 
         # NOTE: might be able to combine contiguous ilocs into a single slice
         iloc_flat: tp.List[GetItemKeyType] = [] # combine into one flat iloc
@@ -381,12 +472,10 @@ class IndexLevel:
         for part in ilocs:
             if isinstance(part, slice):
                 iloc_flat.extend(range(*part.indices(length)))
-            # just look for ints
             elif isinstance(part, INT_TYPES):
                 iloc_flat.append(part)
             else: # assume it is an iterable
-                assert part is not None
-                iloc_flat.extend(part)
+                iloc_flat.extend(part) #type: ignore
         return iloc_flat
 
     #---------------------------------------------------------------------------
@@ -395,7 +484,6 @@ class IndexLevel:
         '''
         Return an immutable NumPy 2D array of all labels found in this IndexLevels instance. This may coerce types.
         '''
-        # assume uniform depths
         depth_count = self.depth
         shape = self.__len__(), depth_count
 
@@ -430,6 +518,37 @@ class IndexLevel:
         labels.flags.writeable = False
         return labels
 
+    def __iter__(self) -> tp.Iterator[tp.Tuple[tp.Hashable, ...]]:
+        # NOTE: this implementation shown to be faster than a recursive purely recursive implementation.
+        depth_count = self.depth
+        levels = deque(((self, 0, None),)) # order matters
+
+        # level: IndexLevel
+        # depth: int
+        # row_previous: tp.Optional[tp.List[tp.Hashable]]
+
+        while levels:
+            level, depth, row_previous = levels.popleft()
+
+            if level.targets is None:
+                for v in level.index.values:
+                    row_previous[depth] = v #type: ignore
+                    yield tuple(row_previous) #type: ignore
+            else: # target is iterable np.ndaarray
+                depth_next = depth + 1
+                for label, level_target in zip(level.index.values, level.targets):
+                    if row_previous is None:
+                        # shown to be faster to allocate entire row width
+                        row = [None] * depth_count
+                    else:
+                        row = row_previous.copy()
+                    row[depth] = label
+                    levels.append((level_target, depth_next, row)) #type: ignore
+
+    # def __iter__(self) -> tp.Iterator[tp.Tuple[tp.Hashable, ...]]:
+    #     part = [None] * self.depth
+    #     yield from _iter_recurse(self, part, 0)
+
     def values_at_depth(self,
             depth_level: int
             ) -> np.ndarray:
@@ -473,6 +592,81 @@ class IndexLevel:
         array.flags.writeable = False
         return array
 
+    #---------------------------------------------------------------------------
+    @doc_inject()
+    def equals(self,
+            other: tp.Any,
+            *,
+            compare_name: bool = False,
+            compare_dtype: bool = False,
+            compare_class: bool = False,
+            skipna: bool = True,
+            ) -> bool:
+        '''
+        {doc}
+
+        Args:
+            {compare_name}
+            {compare_dtype}
+            {compare_class}
+            {skipna}
+        '''
+        if id(other) == id(self):
+            return True
+
+        if compare_class and self.__class__ != other.__class__:
+            return False
+        elif not isinstance(other, IndexLevel):
+            return False
+
+        # same type from here
+        if self.__len__() != other.__len__():
+            return False
+        if self.depth != other.depth:
+            return False
+
+        kwargs = dict(
+                compare_name=compare_name,
+                compare_dtype=compare_dtype,
+                compare_class=compare_class,
+                skipna=skipna,
+                )
+
+        if self.targets is None and other.targets is None:
+            return self.index.equals(other.index, **kwargs) #type: ignore
+
+        # same length and depth, can traverse trees
+        # can store tuple of object ids to note those that have already been examine.
+        equal_pairs = set()
+
+        levels_self = [self]
+        levels_other = [other]
+        while levels_self and levels_other:
+            level_self = levels_self.pop()
+            level_other = levels_other.pop()
+
+            pair = (id(level_self.index), id(level_other.index))
+            pair_found = pair in equal_pairs
+
+            if not pair_found and not level_self.index.equals(level_other.index, **kwargs):
+                return False
+
+            if not pair_found: # but we know it is equal
+                equal_pairs.add(pair)
+
+            if level_self.targets is not None and level_other.targets is not None: # not terminus
+                levels_self.extend(level_self.targets)
+                levels_other.extend(level_other.targets)
+            if level_self.targets is None and level_other.targets is None: # not terminus
+                continue
+            if level_self.targets is None or level_other.targets is None: # not terminus
+                # at least one is at a terminus, but maybe both
+                return False
+
+        if not levels_self and not levels_other:
+            return True # both exhausted
+        return False #pragma: no cover
+
     #---------------------------------------------------------------------------
     # exporters
 
@@ -527,6 +721,7 @@ class IndexLevelGO(IndexLevel):
     _length: tp.Optional[int]
 
     STATIC: bool = False
+    _INDEX_CONSTRUCTOR = IndexGO
 
     #---------------------------------------------------------------------------
     # grow only mutation
