@@ -1,100 +1,94 @@
 import typing as tp
 from functools import partial
+from itertools import chain
 
 import numpy as np
-
 from numpy.ma import MaskedArray
 
-from static_frame.core.util import DEFAULT_SORT_KIND
-from static_frame.core.util import BOOL_TYPES
-from static_frame.core.util import FLOAT_TYPES
-from static_frame.core.util import EMPTY_TUPLE
-
-from static_frame.core.util import GetItemKeyType
-from static_frame.core.util import resolve_dtype
-from static_frame.core.util import isna_array
-from static_frame.core.util import iterable_to_array_1d
-from static_frame.core.util import array_to_groups_and_locations
-from static_frame.core.util import array_to_duplicated
-from static_frame.core.util import full_for_fill
-from static_frame.core.util import mloc
-from static_frame.core.util import immutable_filter
-from static_frame.core.util import name_filter
-from static_frame.core.util import ufunc_axis_skipna
-from static_frame.core.util import array2d_to_tuples
-from static_frame.core.util import array_shift
-from static_frame.core.util import write_optional_file
-from static_frame.core.util import ufunc_unique
-from static_frame.core.util import concat_resolved
-from static_frame.core.util import NULL_SLICE
-from static_frame.core.util import binary_transition
-from static_frame.core.util import isin
-from static_frame.core.util import slices_from_targets
-from static_frame.core.util import is_callable_or_mapping
-
-from static_frame.core.util import AnyCallable
-from static_frame.core.util import CallableOrMapping
-from static_frame.core.util import SeriesInitializer
-from static_frame.core.util import PathSpecifierOrFileLike
-from static_frame.core.util import DepthLevelSpecifier
-
-from static_frame.core.util import DtypeSpecifier
-from static_frame.core.util import IndexInitializer
-from static_frame.core.util import IndexConstructor
-from static_frame.core.util import dtype_to_na
-
-from static_frame.core.selector_node import InterfaceGetItem
-from static_frame.core.selector_node import InterfaceAssignTrio
-from static_frame.core.selector_node import InterfaceSelectTrio
-
-from static_frame.core.util import argmin_1d
-from static_frame.core.util import argmax_1d
-from static_frame.core.util import intersect1d
-
-from static_frame.core.index_correspondence import IndexCorrespondence
-from static_frame.core.container import ContainerOperand
-
-from static_frame.core.display import DisplayConfig
-from static_frame.core.display import DisplayActive
-from static_frame.core.display import Display
-from static_frame.core.display import DisplayFormats
-from static_frame.core.display import DisplayHeader
-
-from static_frame.core.iter_node import IterNodeType
-# from static_frame.core.iter_node import IterNode
-from static_frame.core.iter_node import IterNodeGroup
-from static_frame.core.iter_node import IterNodeDepthLevel
-from static_frame.core.iter_node import IterNodeWindow
-from static_frame.core.iter_node import IterNodeNoArg
-
-from static_frame.core.iter_node import IterNodeApplyType
-
-from static_frame.core.index import Index
-
-# from static_frame.core.index_hierarchy import HLoc
-from static_frame.core.index_hierarchy import IndexHierarchy
-from static_frame.core.index_base import IndexBase
-
-from static_frame.core.container_util import index_from_optional_constructor
-from static_frame.core.container_util import matmul
-from static_frame.core.container_util import axis_window_items
-from static_frame.core.container_util import rehierarch_and_map
-from static_frame.core.container_util import pandas_version_under_1
-from static_frame.core.container_util import pandas_to_numpy
-
-from static_frame.core.index_auto import IndexAutoFactory
-from static_frame.core.index_auto import IndexAutoFactoryType
 
 from static_frame.core.assign import Assign
 
-from static_frame.core.exception import ErrorInitSeries
-from static_frame.core.exception import AxisInvalid
+from static_frame.core.container import ContainerOperand
+from static_frame.core.container_util import apply_binary_operator
+from static_frame.core.container_util import axis_window_items
+from static_frame.core.container_util import index_from_optional_constructor
+from static_frame.core.container_util import matmul
+from static_frame.core.container_util import pandas_to_numpy
+from static_frame.core.container_util import pandas_version_under_1
+from static_frame.core.container_util import rehierarch_from_index_hierarchy
 
+from static_frame.core.display import Display
+from static_frame.core.display import DisplayActive
+from static_frame.core.display import DisplayConfig
+from static_frame.core.display import DisplayFormats
+from static_frame.core.display import DisplayHeader
 from static_frame.core.doc_str import doc_inject
+from static_frame.core.exception import AxisInvalid
+from static_frame.core.exception import ErrorInitSeries
+
+from static_frame.core.index import Index
+from static_frame.core.index_auto import IndexAutoFactory
+from static_frame.core.index_auto import IndexAutoFactoryType
+from static_frame.core.index_base import IndexBase
+from static_frame.core.index_correspondence import IndexCorrespondence
+from static_frame.core.index_hierarchy import IndexHierarchy
+
+from static_frame.core.node_dt import InterfaceDatetime
+from static_frame.core.node_iter import IterNodeApplyType
+from static_frame.core.node_iter import IterNodeDepthLevel
+from static_frame.core.node_iter import IterNodeGroup
+from static_frame.core.node_iter import IterNodeNoArg
+from static_frame.core.node_iter import IterNodeType
+from static_frame.core.node_iter import IterNodeWindow
+from static_frame.core.node_selector import InterfaceAssignTrio
+from static_frame.core.node_selector import InterfaceGetItem
+from static_frame.core.node_selector import InterfaceSelectTrio
+from static_frame.core.node_str import InterfaceString
+
+from static_frame.core.util import AnyCallable
+from static_frame.core.util import argmax_1d
+from static_frame.core.util import argmin_1d
+from static_frame.core.util import array_shift
+from static_frame.core.util import array_to_duplicated
+from static_frame.core.util import array_to_groups_and_locations
+from static_frame.core.util import array2d_to_tuples
+from static_frame.core.util import binary_transition
+from static_frame.core.util import CallableOrMapping
+from static_frame.core.util import concat_resolved
+from static_frame.core.util import DEFAULT_SORT_KIND
+from static_frame.core.util import DepthLevelSpecifier
+from static_frame.core.util import dtype_to_na
+from static_frame.core.util import DtypeSpecifier
+from static_frame.core.util import EMPTY_TUPLE
+from static_frame.core.util import FLOAT_TYPES
+from static_frame.core.util import full_for_fill
+from static_frame.core.util import GetItemKeyType
+from static_frame.core.util import immutable_filter
+from static_frame.core.util import IndexConstructor
+from static_frame.core.util import IndexInitializer
+from static_frame.core.util import INT_TYPES
+from static_frame.core.util import intersect1d
+from static_frame.core.util import is_callable_or_mapping
+from static_frame.core.util import isin
+from static_frame.core.util import isna_array
+from static_frame.core.util import iterable_to_array_1d
+from static_frame.core.util import mloc
+from static_frame.core.util import NAME_DEFAULT
+from static_frame.core.util import name_filter
+from static_frame.core.util import NameType
+from static_frame.core.util import NULL_SLICE
+from static_frame.core.util import PathSpecifierOrFileLike
+from static_frame.core.util import resolve_dtype
+from static_frame.core.util import SeriesInitializer
+from static_frame.core.util import slices_from_targets
+from static_frame.core.util import ufunc_axis_skipna
+from static_frame.core.util import ufunc_unique
+from static_frame.core.util import write_optional_file
 
 if tp.TYPE_CHECKING:
     from static_frame import Frame # pylint: disable=W0611 #pragma: no cover
-    from pandas import DataFrame # pylint: disable=W0611 #pragma: no cover
+    from static_frame import FrameGO # pylint: disable=W0611 #pragma: no cover
+    import pandas # pylint: disable=W0611 #pragma: no cover
 
 
 RelabelInput = tp.Union[CallableOrMapping, IndexAutoFactoryType, IndexInitializer]
@@ -339,7 +333,7 @@ class Series(ContainerOperand):
             values: SeriesInitializer,
             *,
             index: tp.Union[IndexInitializer, IndexAutoFactoryType, None] = None,
-            name: tp.Optional[tp.Hashable] = None,
+            name: NameType = NAME_DEFAULT,
             dtype: DtypeSpecifier = None,
             index_constructor: IndexConstructor = None,
             own_index: bool = False
@@ -347,14 +341,13 @@ class Series(ContainerOperand):
         '''
         Default constructor of :obj:`static_frame.Series`.
         '''
-        # doc string at class definition
-        self._name = name if name is None else name_filter(name)
 
         if own_index and index is None:
             raise ErrorInitSeries('cannot own_index if no index is provided.')
 
         #-----------------------------------------------------------------------
         # values assignment
+
         values_constructor = None # if deferred
 
         if not isinstance(values, np.ndarray):
@@ -368,6 +361,8 @@ class Series(ContainerOperand):
                     # set up for direct assignment below; index is always immutable
                     index = values.index
                     own_index = True
+                if name is NAME_DEFAULT:
+                    name = values.name # propagate Series.name
             elif hasattr(values, '__iter__') and not isinstance(values, str):
                 # returned array is already immutable
                 self.values, _ = iterable_to_array_1d(values, dtype=dtype)
@@ -385,6 +380,8 @@ class Series(ContainerOperand):
             else:
                 self.values = immutable_filter(values)
 
+        self._name = None if name is NAME_DEFAULT else name_filter(name)
+
         #-----------------------------------------------------------------------
         # index assignment
 
@@ -454,7 +451,7 @@ class Series(ContainerOperand):
         '''{}'''
         return self._name
 
-    def rename(self, name: tp.Hashable) -> 'Series':
+    def rename(self, name: NameType) -> 'Series':
         '''
         Return a new Series with an updated name attribute.
         '''
@@ -480,8 +477,6 @@ class Series(ContainerOperand):
         '''
         return InterfaceGetItem(self._extract_iloc)
 
-    # NOTE: this could be ExtractInterfacd1D, but are consistent with what is done on the base name space: loc and getitem duplicate each other.
-
     @property
     def drop(self) -> InterfaceSelectTrio:
         '''
@@ -528,6 +523,47 @@ class Series(ContainerOperand):
                 delegate=SeriesAssign
                 )
 
+    #---------------------------------------------------------------------------
+    @property
+    def via_str(self) -> InterfaceString['Series']:
+        '''
+        Interface for applying string methods to elements in this container.
+        '''
+        blocks = (self.values,)
+
+        def blocks_to_container(blocks: tp.Iterator[np.ndarray]) -> 'Frame':
+            return self.__class__(
+                next(blocks), # assume only one
+                index=self._index,
+                name=self._name,
+                own_index=True,
+                )
+
+        return InterfaceString(
+                blocks=blocks,
+                blocks_to_container=blocks_to_container,
+                )
+
+    @property
+    def via_dt(self) -> InterfaceDatetime['Series']:
+        '''
+        Interface for applying datetime properties and methods to elements in this container.
+        '''
+        blocks = (self.values,)
+
+        def blocks_to_container(blocks: tp.Iterator[np.ndarray]) -> 'Frame':
+            return self.__class__(
+                next(blocks), # assume only one
+                index=self._index,
+                name=self._name,
+                own_index=True,
+                )
+
+        return InterfaceDatetime(
+                blocks=blocks,
+                blocks_to_container=blocks_to_container,
+                )
+
     #---------------------------------------------------------------------------
     @property
     def iter_group(self) -> IterNodeGroup:
@@ -647,7 +683,7 @@ class Series(ContainerOperand):
     def _reindex_other_like_iloc(self,
             value: 'Series',
             iloc_key: GetItemKeyType,
-            fill_value=np.nan) -> 'Series':
+            fill_value: tp.Any = np.nan) -> 'Series':
         '''Given a value that is a Series, reindex it to the index components, drawn from this Series, that are specified by the iloc_key.
         '''
         return value.reindex(
@@ -658,8 +694,10 @@ class Series(ContainerOperand):
     @doc_inject(selector='reindex', class_name='Series')
     def reindex(self,
             index: IndexInitializer,
+            *,
             fill_value=np.nan,
-            own_index: bool = False
+            own_index: bool = False,
+            check_equals: bool = True
             ) -> 'Series':
         '''
         {doc}
@@ -674,10 +712,22 @@ class Series(ContainerOperand):
             index = index_from_optional_constructor(index,
                     default_constructor=Index)
 
-        ic = IndexCorrespondence.from_correspondence(self.index, index)
+        # NOTE: it is assumed that the equals comparison is faster than continuing with this method
+        if check_equals and self._index.equals(index):
+            # if labels are equal (even if a different Index subclass), we can simply use the new Index
+            return self.__class__(
+                    self.values,
+                    index=index,
+                    own_index=True,
+                    name=self._name)
+
+        ic = IndexCorrespondence.from_correspondence(self._index, index)
 
         if ic.is_subset: # must have some common
-            return self.__class__(self.values[ic.iloc_src],
+            values = self.values[ic.iloc_src]
+            values.flags.writeable = False
+            return self.__class__(
+                    values,
                     index=index,
                     own_index=True,
                     name=self._name)
@@ -766,12 +816,10 @@ class Series(ContainerOperand):
         if self.index.depth == 1:
             raise RuntimeError('cannot rehierarch when there is no hierarchy')
 
-        # index, iloc_map = self.index._rehierarch_and_map(depth_map=depth_map)
-
-        index, iloc_map = rehierarch_and_map(
-                labels=self._index.values,
+        index, iloc_map = rehierarch_from_index_hierarchy(
+                labels=self._index,
                 depth_map=depth_map,
-                index_constructor=self._index.from_labels,
+                name=self._index.name,
                 )
         values = self.values[iloc_map]
         values.flags.writeable = False
@@ -1017,10 +1065,10 @@ class Series(ContainerOperand):
 
     def _ufunc_binary_operator(self, *,
             operator: tp.Callable,
-            other
+            other: tp.Any,
             ) -> 'Series':
         '''
-        For binary operations, the `name` attribute does not propagate.
+        For binary operations, the `name` attribute does not propagate unless other is a scalar.
         '''
         # get both reverse and regular
         if operator.__name__ == 'matmul':
@@ -1030,37 +1078,34 @@ class Series(ContainerOperand):
 
         values = self.values
         index = self._index
+        other_is_array = False
 
         if isinstance(other, Series):
-            # if indices are the same, we can simply set other to values and fallback on NP
-            if len(self.index) != len(other.index) or (
-                    self.index != other.index).any():
-                index = self.index.union(other.index)
+            name = None
+            other_is_array = True
+            if not self._index.equals(other._index):
+                # if not equal, create a new Index by forming the union
+                index = self._index.union(other._index)
                 # now need to reindex the Series
-                values = self.reindex(index).values
-                other = other.reindex(index).values
+                values = self.reindex(index, own_index=True, check_equals=False).values
+                other = other.reindex(index, own_index=True, check_equals=False).values
             else:
                 other = other.values
-
-        # if its an np array, we simply fall back on np behavior
         elif isinstance(other, np.ndarray):
+            name = None
+            other_is_array = True
             if other.ndim > 1:
-                raise NotImplementedError('Operator application to greater dimensionalities will result in an array with more than 1 dimension; it is not clear how such an array should be indexed.')
-        # permit single value constants; not sure about filtering other types
+                raise NotImplementedError('Operator application to greater dimensionalities will result in an array with more than 1 dimension.')
+        else:
+            name = self._name
 
-        # we want the dtype to be the result of applying the operator; this happends by default
-        result = operator(values, other)
-
-        if not isinstance(result, np.ndarray):
-            # in comparison to Booleans, if values is of length 1 and a character type, we will get a Boolean back, not an array; this issues the following warning: FutureWarning: elementwise comparison failed; returning scalar instead, but in the future will perform elementwise comparison
-            if isinstance(result, BOOL_TYPES):
-                # return a Boolean at the same size as the original Series; this works, but means that we will mask that, if the arguement is a tuple of length equal to an erray, NP will perform element wise comparison; but if the argment is a tuple of length greater or equal, each value in value will be compared to that tuple
-                result = np.full(len(values), result)
-            else:
-                raise RuntimeError('unexpected branch from non-array result of operator application to array') #pragma: no cover
-
-        result.flags.writeable = False
-        return self.__class__(result, index=index)
+        result = apply_binary_operator(
+                values=values,
+                other=other,
+                other_is_array=other_is_array,
+                operator=operator,
+                )
+        return self.__class__(result, index=index, name=name)
 
     def _ufunc_axis_skipna(self, *,
             axis: int,
@@ -1116,6 +1161,43 @@ class Series(ContainerOperand):
         '''
         return self.values.__len__()
 
+    def _display(self,
+            config: DisplayConfig,
+            display_cls: Display,
+            ) -> Display:
+        '''
+        Private display interface to be shared by Bus and Series.
+        '''
+        index_depth = self._index.depth if config.include_index else 0
+        display_index = self._index.display(config=config)
+
+        # When showing type we need 2: one for the Series type, the other for the index type.
+        header_depth = 2 * config.type_show
+
+        # create an empty display based on index display
+        d = Display([list() for _ in range(len(display_index))],
+                config=config,
+                outermost=True,
+                index_depth=index_depth,
+                header_depth=header_depth
+                )
+
+        if config.include_index:
+            d.extend_display(display_index)
+            header_values = '' if config.type_show else None
+        else:
+            header_values = None
+
+        d.extend_display(Display.from_values(
+                self.values,
+                header=header_values,
+                config=config))
+
+        if config.type_show:
+            d.insert_displays(display_cls.flatten())
+
+        return d
+
     @doc_inject()
     def display(self,
             config: tp.Optional[DisplayConfig] = None
@@ -1126,37 +1208,44 @@ class Series(ContainerOperand):
             {config}
         '''
         config = config or DisplayActive.get()
-
-        d = Display([],
-                config=config,
-                outermost=True,
-                index_depth=1,
-                header_depth=2) # series and index header
-
-        display_index = self._index.display(config=config)
-        d.extend_display(display_index)
-
-        d.extend_display(Display.from_values(
-                self.values,
-                header='',
-                config=config))
-
         display_cls = Display.from_values((),
                 header=DisplayHeader(self.__class__, self._name),
                 config=config)
-        d.insert_displays(display_cls.flatten())
-        return d
+        return self._display(config, display_cls)
 
-    def _repr_html_(self):
-        '''
-        Provide HTML representation for Jupyter Notebooks.
-        '''
-        # modify the active display to be fore HTML
-        config = DisplayActive.get(
-                display_format=DisplayFormats.HTML_TABLE,
-                type_show=False
-                )
-        return repr(self.display(config))
+        # config = config or DisplayActive.get()
+        # index_depth = self._index.depth if config.include_index else 0
+        # display_index = self._index.display(config=config)
+
+        # # When showing type we need 2: one for the Series type, the other for the index type.
+        # header_depth = 2 * config.type_show
+
+        # # create an empty display based on index display
+        # d = Display([list() for _ in range(len(display_index))],
+        #         config=config,
+        #         outermost=True,
+        #         index_depth=index_depth,
+        #         header_depth=header_depth
+        #         )
+
+        # if config.include_index:
+        #     d.extend_display(display_index)
+        #     header_values = '' if config.type_show else None
+        # else:
+        #     header_values = None
+
+        # d.extend_display(Display.from_values(
+        #         self.values,
+        #         header=header_values,
+        #         config=config))
+
+        # if config.type_show:
+        #     display_cls = Display.from_values((),
+        #             header=DisplayHeader(self.__class__, self._name),
+        #             config=config)
+        #     d.insert_displays(display_cls.flatten())
+
+        # return d
 
     #---------------------------------------------------------------------------
     # common attributes from the numpy array
@@ -1338,7 +1427,7 @@ class Series(ContainerOperand):
 
     def _axis_group_items(self, *,
             axis: int = 0
-            ):
+            ) -> tp.Iterator[tp.Tuple[tp.Hashable, 'Series']]:
         if axis != 0:
             raise AxisInvalid(f'invalid axis {axis}')
 
@@ -1349,7 +1438,7 @@ class Series(ContainerOperand):
 
     def _axis_group(self, *,
             axis: int = 0
-            ):
+            ) -> tp.Iterator['Series']:
         yield from (x for _, x in self._axis_group_items(axis=axis))
 
 
@@ -1636,8 +1725,9 @@ class Series(ContainerOperand):
 
     @doc_inject(selector='duplicated')
     def duplicated(self, *,
-            exclude_first=False,
-            exclude_last=False) -> np.ndarray:
+            exclude_first: bool = False,
+            exclude_last: bool = False,
+            ) -> np.ndarray:
         '''
         Return a same-sized Boolean Series that shows True for all b values that are duplicated.
 
@@ -1695,10 +1785,9 @@ class Series(ContainerOperand):
                 name=self._name
                 )
 
-
-    def __round__(self, decimals: int = 0):
+    def __round__(self, decimals: int = 0) -> 'Series':
         '''
-        Return a Series rounded to the given decimals.
+        Return a Series rounded to the given decimals. Negative decimals round to the left of the decimal point.
 
         Args:
             decimals: number of decimals to round to.
@@ -1751,7 +1840,7 @@ class Series(ContainerOperand):
     def shift(self,
             shift: int,
             *,
-            fill_value=np.nan) -> 'Series':
+            fill_value: tp.Any = np.nan) -> 'Series':
         '''Return a Series with values shifted forward on the index (with a postive shift) or backward on the index (with a negative shift).
 
         Args:
@@ -1779,7 +1868,8 @@ class Series(ContainerOperand):
 
 
     #---------------------------------------------------------------------------
-    # transformations resulting in reduced dimensionality
+    # transformations resulting in changed dimensionality
+
     @doc_inject(selector='head', class_name='Series')
     def head(self, count: int = 5) -> 'Series':
         '''{doc}
@@ -1871,9 +1961,89 @@ class Series(ContainerOperand):
         '''
         return argmax_1d(self.values, skipna=skipna)
 
+    #---------------------------------------------------------------------------
+    def _insert(self,
+            key: int, # iloc positions
+            container: 'Series',
+            ) -> 'Series':
+        if not isinstance(container, Series):
+            raise NotImplementedError(
+                    f'No support for inserting with {type(container)}')
+
+        if not len(container.index): # must be empty data, empty index container
+            return self
+
+        dtype = resolve_dtype(self.values.dtype, container.dtype)
+        values = np.empty(len(self) + len(container), dtype=dtype)
+        key_end = key + len(container)
+
+        values_prior = self.values
+
+        values[:key] = values_prior[:key]
+        values[key: key_end] = container.values
+        values[key_end:] = values_prior[key:]
+        values.flags.writeable = False
+
+        labels_prior = self._index.values
+
+        index = self._index.__class__.from_labels(chain(
+                labels_prior[:key],
+                container._index.__iter__(),
+                labels_prior[key:],
+                ))
+
+        return self.__class__(values,
+                index=index,
+                name=self._name,
+                own_index=True,
+                )
+
+    @doc_inject(selector='insert')
+    def insert_before(self,
+            key: tp.Hashable,
+            container: 'Series',
+            ) -> 'Series':
+        '''
+        Create a new :obj:`Series` by inserting a :obj:`Series` at the position before the label specified by ``key``.
+
+        Args:
+            {key_before}
+            {container}
+
+        Returns:
+            :obj:`Series`
+        '''
+        iloc_key = self._index.loc_to_iloc(key)
+        if not isinstance(iloc_key, INT_TYPES):
+            raise RuntimeError(f'Unsupported key type: {key}')
+        return self._insert(iloc_key, container)
+
+    @doc_inject(selector='insert')
+    def insert_after(self,
+            key: tp.Hashable, # iloc positions
+            container: 'Series',
+            ) -> 'Series':
+        '''
+        Create a new :obj:`Series` by inserting a :obj:`Series` at the position after the label specified by ``key``.
+
+        Args:
+            {key_after}
+            {container}
+
+        Returns:
+            :obj:`Series`
+        '''
+        iloc_key = self._index.loc_to_iloc(key)
+        if not isinstance(iloc_key, INT_TYPES):
+            raise RuntimeError(f'Unsupported key type: {key}')
+        return self._insert(iloc_key + 1, container)
+
+
+
+
 
     #---------------------------------------------------------------------------
-    # utility function to numpy array
+    # utility function to numpy array or other types
 
     def unique(self) -> np.ndarray:
         '''
@@ -1884,6 +2054,61 @@ class Series(ContainerOperand):
         '''
         return ufunc_unique(self.values)
 
+    @doc_inject()
+    def equals(self,
+            other: tp.Any,
+            *,
+            compare_name: bool = False,
+            compare_dtype: bool = False,
+            compare_class: bool = False,
+            skipna: bool = True,
+            ) -> bool:
+        '''
+        {doc}
+
+        Args:
+            {compare_name}
+            {compare_dtype}
+            {compare_class}
+            {skipna}
+        '''
+        if id(other) == id(self):
+            return True
+
+        # NOTE: there are presently no Series subclasses, but better to be consistent
+        if compare_class and self.__class__ != other.__class__:
+            return False
+        elif not isinstance(other, Series):
+            return False
+
+        if len(self.values) != len(other.values):
+            return False
+        if compare_name and self._name != other._name:
+            return False
+        if compare_dtype and self.values.dtype != other.values.dtype:
+            return False
+
+        eq = self.values == other.values
+
+        # NOTE: will only be False, or an array
+        if eq is False:
+            return eq
+
+        if skipna:
+            isna_both = (isna_array(self.values, include_none=False) &
+                    isna_array(other.values, include_none=False))
+            eq[isna_both] = True
+
+        if not eq.all():
+            return False
+
+        return self._index.equals(other._index,
+                compare_name=compare_name,
+                compare_dtype=compare_dtype,
+                compare_class=compare_class,
+                skipna=skipna,
+                )
+
     #---------------------------------------------------------------------------
     # export
 
@@ -1904,9 +2129,9 @@ class Series(ContainerOperand):
 
 
     def _to_frame(self,
-            constructor: tp.Type[ContainerOperand],
+            constructor: tp.Type['Frame'],
             axis: int = 1
-            ):
+            ) -> 'Frame':
         '''
         Common Frame construction utilities.
         '''
@@ -1914,7 +2139,7 @@ class Series(ContainerOperand):
 
         if axis == 1:
             # present as a column
-            def block_gen():
+            def block_gen() -> tp.Iterator[np.ndarray]:
                 yield self.values
 
             index = self._index
@@ -1922,7 +2147,7 @@ class Series(ContainerOperand):
             columns = None if self._name is None else (self._name,)
             own_columns = False
         elif axis == 0:
-            def block_gen():
+            def block_gen() -> tp.Iterator[np.ndarray]:
                 yield self.values.reshape((1, self.values.shape[0]))
 
             index = None if self._name is None else (self._name,)
@@ -1936,7 +2161,7 @@ class Series(ContainerOperand):
         return constructor(
                 TypeBlocks.from_blocks(block_gen()),
                 index=index,
-                columns=columns,
+                columns=columns, #type: ignore
                 own_data=True,
                 own_index=own_index,
                 own_columns=own_columns,
@@ -1961,9 +2186,9 @@ class Series(ContainerOperand):
             :obj:`static_frame.FrameGO`
         '''
         from static_frame import FrameGO
-        return self._to_frame(constructor=FrameGO, axis=axis)
+        return self._to_frame(constructor=FrameGO, axis=axis) #type: ignore
 
-    def to_pandas(self) -> 'pd.Series':
+    def to_pandas(self) -> 'pandas.Series':
         '''
         Return a Pandas Series.
 
@@ -1978,7 +2203,7 @@ class Series(ContainerOperand):
     @doc_inject(class_name='Series')
     def to_html(self,
             config: tp.Optional[DisplayConfig] = None
-            ):
+            ) -> str:
         '''
         {}
         '''
@@ -2006,6 +2231,7 @@ class Series(ContainerOperand):
         fp = write_optional_file(content=content, fp=fp)
 
         if show:
+            assert isinstance(fp, str) #pragma: no cover
             import webbrowser #pragma: no cover
             webbrowser.open_new_tab(fp) #pragma: no cover
         return fp
@@ -2023,9 +2249,9 @@ class SeriesAssign(Assign):
         self.iloc_key = iloc_key
 
     def __call__(self,
-            value, # any possible assignment type
-            fill_value=np.nan
-            ):
+            value: tp.Any, # any possible assignment type
+            fill_value: tp.Any = np.nan
+            ) -> Series:
         '''
         Assign the ``value`` in the position specified by the selector. The `name` attribute is propagated to the returned container.
 
