@@ -3,28 +3,21 @@
 import typing as tp
 import os
 
-
-
 from itertools import chain
 from functools import partial
 from functools import wraps
 import numpy as np
 
-from static_frame.core.util import AnyCallable
 
-
-from static_frame.core.frame import Frame
 from static_frame.core.exception import ErrorInitStore
 from static_frame.core.exception import ErrorInitStoreConfig
 from static_frame.core.exception import StoreFileMutation
-
-from static_frame.core.util import PathSpecifier
-from static_frame.core.util import path_filter
-# from static_frame.core.util import array2d_to_tuples
+from static_frame.core.frame import Frame
 from static_frame.core.index_hierarchy import IndexHierarchy
+from static_frame.core.util import AnyCallable
 from static_frame.core.util import DtypesSpecifier
-
-
+from static_frame.core.util import path_filter
+from static_frame.core.util import PathSpecifier
 
 
 #-------------------------------------------------------------------------------
@@ -331,7 +324,7 @@ class Store:
             ) -> Frame:
         '''Read a single Frame, given by `label`, from the Store. Return an instance of `container_type`.
         '''
-        raise NotImplementedError()
+        raise NotImplementedError() #pragma: no cover
 
     def write(self,
             items: tp.Iterable[tp.Tuple[str, Frame]],
@@ -340,10 +333,10 @@ class Store:
             ) -> None:
         '''Write all ``Frames`` in the Store.
         '''
-        raise NotImplementedError()
+        raise NotImplementedError() #pragma: no cover
 
     def labels(self, strip_ext: bool = True) -> tp.Iterator[str]:
-        raise NotImplementedError()
+        raise NotImplementedError() #pragma: no cover
 
 
 
