@@ -1,19 +1,20 @@
-import typing as tp
-import os
-import operator
 
 from collections import abc
 from collections import defaultdict
 from collections import namedtuple
+from enum import Enum
 from functools import partial
 from functools import reduce
+from io import StringIO
 from itertools import chain
 from itertools import zip_longest
-from io import StringIO
-import datetime
-from urllib import request
-import tempfile
 from pathlib import Path
+from urllib import request
+import datetime
+import operator
+import os
+import tempfile
+import typing as tp
 
 from automap import FrozenAutoMap  # pylint: disable = E0611
 import numpy as np
@@ -231,12 +232,30 @@ FrameInitializer = tp.Union[
         ] # need to add FRAME_INITIALIZER_DEFAULT
 
 FILL_VALUE_DEFAULT = object()
+NAME_DEFAULT = object()
 
 
 DateInitializer = tp.Union[str, datetime.date, np.datetime64]
 YearMonthInitializer = tp.Union[str, datetime.date, np.datetime64]
 YearInitializer = tp.Union[str, datetime.date, np.datetime64]
 
+#-------------------------------------------------------------------------------
+# join utils
+
+class Join(Enum):
+    INNER = 0
+    LEFT = 1
+    RIGHT = 2
+    OUTER = 3
+
+class Pair(tuple): #type: ignore
+    pass
+
+class PairLeft(Pair):
+    pass
+
+class PairRight(Pair):
+    pass
 
 #-------------------------------------------------------------------------------
 
@@ -276,7 +295,7 @@ def immutable_filter(src_array: np.ndarray) -> np.ndarray:
         return dst_array
     return src_array # keep it as is
 
-def name_filter(name: tp.Hashable) -> tp.Hashable:
+def name_filter(name: NameType) -> NameType:
     '''
     For name attributes on containers, only permit recursively hashable objects.
     '''
@@ -736,8 +755,7 @@ def resolve_type_iter(
     is_gen, copy_values = is_gen_copy_values(values)
 
     if not is_gen and len(values) == 0: #type: ignore
-        values = tp.cast(tp.Sequence[tp.Any], values)
-        return None, False, values
+        return None, False, values #type: ignore
 
     if restrict_copy:
         copy_values = False
@@ -750,6 +768,7 @@ def resolve_type_iter(
     resolved = None # None is valid specifier if the type is not ambiguous
     has_tuple = False
     has_str = False
+    has_enum = False
     has_non_str = False
     has_inexact = False
     has_big_int = False
@@ -766,6 +785,9 @@ def resolve_type_iter(
             # need to get tuple subclasses, like NamedTuple
             if isinstance(v, (tuple, list)):
                 has_tuple = True
+            elif isinstance(v, Enum):
+                # must check isinstance, as Enum types are alwyas derived from Enum
+                has_enum = True
             elif value_type == str or value_type == np.str_:
                 # must compare to both sring types
                 has_str = True
@@ -776,7 +798,7 @@ def resolve_type_iter(
                 elif value_type == int and abs(v) > INT_MAX_COERCIBLE_TO_FLOAT:
                     has_big_int = True
 
-            if has_tuple or (has_str and has_non_str):
+            if has_tuple or has_enum or (has_str and has_non_str):
                 resolved = object
             elif has_big_int and has_inexact:
                 resolved = object
@@ -807,7 +829,7 @@ def iterable_to_array_1d(
     '''
     if isinstance(values, np.ndarray):
         if dtype is not None and dtype != values.dtype:
-            raise RuntimeError(f'supplied dtype {dtype} not set on supplied array')
+            raise RuntimeError(f'Supplied dtype {dtype} not set on supplied array.')
         return values, len(values) <= 1
 
     if isinstance(values, range):
@@ -819,6 +841,9 @@ def iterable_to_array_1d(
         array.flags.writeable = False
         return array, True
 
+    if hasattr(values, 'values') and hasattr(values, 'index'):
+        raise RuntimeError(f'Supplied iterable {type(values)} appears to be labeled, though labels are not being used. Convert to a Series.')
+
     values_for_construct: tp.Sequence[tp.Any]
 
     # values for construct will only be a copy when necessary in iteration to find type
@@ -831,7 +856,7 @@ def iterable_to_array_1d(
         dtype, has_tuple, values_for_construct = resolve_type_iter(values)
         if len(values_for_construct) == 0:
             return EMPTY_ARRAY, True # no dtype given, so return empty float array
-    else: # dtype given, do not do full iteration
+    else:
         is_gen, copy_values = is_gen_copy_values(values)
         if copy_values:
             # we have to realize into sequence for numpy creation
@@ -874,7 +899,6 @@ def iterable_to_array_1d(
     return v, is_unique
 
 
-
 def iterable_to_array_2d(
         values: tp.Iterable[tp.Iterable[tp.Any]],
         ) -> np.ndarray:
@@ -904,6 +928,28 @@ def iterable_to_array_2d(
     array.flags.writeable = False
     return array
 
+def iterable_to_array_nd(
+        values: tp.Any,
+        ) -> np.ndarray:
+    '''
+    Attempt to determiine if a value is 0, 1, or 2D array; this will interpret lists of tuples as 2D, as NumPy does.
+    '''
+    if hasattr(values, '__iter__') and not isinstance(values, str):
+
+        values = iter(values)
+        try:
+            first = next(values)
+        except StopIteration:
+            return EMPTY_ARRAY
+
+        if hasattr(first, '__iter__') and not isinstance(first, str):
+            return iterable_to_array_2d(chain((first,), values))
+
+        array, _ = iterable_to_array_1d(chain((first,), values))
+        return array
+    # its an element
+    return np.array(values)
+
 #-------------------------------------------------------------------------------
 
 def slice_to_ascending_slice(
@@ -953,6 +999,9 @@ DT64_S = np.dtype('datetime64[s]')
 DT64_MS = np.dtype('datetime64[ms]')
 DT64_US = np.dtype('datetime64[us]')
 DT64_NS = np.dtype('datetime64[ns]')
+DT64_PS = np.dtype('datetime64[ps]')
+DT64_FS = np.dtype('datetime64[fs]')
+DT64_AS = np.dtype('datetime64[as]')
 
 TD64_YEAR = np.timedelta64(1, 'Y')
 TD64_MONTH = np.timedelta64(1, 'M')
@@ -1017,12 +1066,16 @@ def _slice_to_datetime_slice_args(key: slice,
         value = getattr(key, attr)
         if value is None:
             yield None
+        elif attr == SLICE_STEP_ATTR:
+            # steps are never transformed
+            yield value
         else:
             yield to_datetime64(value, dtype=dtype)
 
 def key_to_datetime_key(
         key: GetItemKeyType,
-        dtype: np.dtype = np.datetime64) -> GetItemKeyType:
+        dtype: np.dtype = np.datetime64,
+        ) -> GetItemKeyType:
     '''
     Given an get item key for a Date index, convert it to np.datetime64 representation.
     '''
@@ -1092,7 +1145,9 @@ def isna_element(value: tp.Any) -> bool:
     return value is None
 
 
-def isna_array(array: np.ndarray) -> np.ndarray:
+def isna_array(array: np.ndarray,
+        include_none: bool = True,
+        ) -> np.ndarray:
     '''Given an np.ndarray, return a bolean array setting True for missing values.
 
     Note: the returned array is not made immutable.
@@ -1108,24 +1163,9 @@ def isna_array(array: np.ndarray) -> np.ndarray:
         return np.full(array.shape, False, dtype=bool)
     # only check for None if we have an object type
     # NOTE: this will not work for Frames contained within a Series
-    return np.not_equal(array, array) | np.equal(array, None)
-
-    # try: # this will only work for arrays that do not have strings
-    #     # astype: None gets converted to nan if possible
-    #     # cannot use can_cast to reliabily identify arrays with non-float-castable elements
-    #     return np.isnan(array.astype(float))
-    # except ValueError:
-    #     # this Exception means there was a character or something not castable to float
-    #     # this is a big perforamnce hit; problem is cannot find np.nan in numpy object array
-    #     if array.ndim == 1:
-    #         return np.fromiter((x is None or x is np.nan for x in array),
-    #                 count=array.size,
-    #                 dtype=bool)
-
-    #     return np.fromiter((x is None or x is np.nan for x in array.flat),
-    #             count=array.size,
-    #             dtype=bool).reshape(array.shape)
-
+    if include_none:
+        return np.not_equal(array, array) | np.equal(array, None)
+    return np.not_equal(array, array)
 
 
 def binary_transition(
@@ -1476,12 +1516,17 @@ def _ufunc_set_1d(
             result = frozenset(array) & frozenset(other)
         else:
             result = frozenset(array).difference(frozenset(other))
+        # NOTE: try to sort, as set ordering is not stable
+        try:
+            result = sorted(result) #type: ignore
+        except TypeError:
+            pass
         v, _ = iterable_to_array_1d(result, dtype)
         return v
 
     if is_union:
         return func(array, other)
-    return func(array, other, assume_unique=assume_unique) # type: ignore (Callable doesn't support optional kwargs)
+    return func(array, other, assume_unique=assume_unique) #type: ignore
 
 def _ufunc_set_2d(
         func: tp.Callable[[np.ndarray, np.ndarray], np.ndarray],
@@ -1499,11 +1544,14 @@ def _ufunc_set_2d(
         other: can be a 2D array, or a 1D object array of tuples.
         assume_unique: if True, array operands are assumed unique and order is preserved for matching operands.
     Returns:
-        Either a 2D array, or a 1D object array of tuples.
+        Either a 2D array (if both operands are 2D), or a 1D object array of tuples (if one or both are 1d tuple arrays).
     '''
+    # NOTE: diversity if ruturned values may be a problem; likely should always return 2D array, or follow pattern that if both operands are 2D, a 2D array is returned
+
     is_union = func == np.union1d
     is_intersection = func == np.intersect1d
     is_difference = func == np.setdiff1d
+    is_2d = array.ndim == 2 and other.ndim == 2
 
     if not (is_union or is_intersection or is_difference):
         raise NotImplementedError('unexpected func', func)
@@ -1514,10 +1562,13 @@ def _ufunc_set_2d(
     # optimizations for empty arrays
     if is_intersection: # intersection with empty
         if len(array) == 0 or len(other) == 0:
-            # not sure what DTYPE is correct to return here
+            if is_2d:
+                return np.array(EMPTY_TUPLE, dtype=dtype).reshape(0, 0)
             return np.array(EMPTY_TUPLE, dtype=dtype)
     elif is_difference:
         if len(array) == 0:
+            if is_2d:
+                return np.array(EMPTY_TUPLE, dtype=dtype).reshape(0, 0)
             return np.array(EMPTY_TUPLE, dtype=dtype)
 
     if assume_unique:
@@ -1534,18 +1585,17 @@ def _ufunc_set_2d(
         if array.shape == other.shape:
             arrays_are_equal = False
             compare = array == other
-
             # will not match a 2D array of integers and 1D array of tuples containing integers (would have to do a post-set comparison, but would loose order)
             if isinstance(compare, BOOL_TYPES) and compare:
                 arrays_are_equal = True #pragma: no cover
             elif isinstance(compare, np.ndarray) and compare.all(axis=None):
                 arrays_are_equal = True
-
             if arrays_are_equal:
                 if is_difference:
+                    if is_2d:
+                        return np.array(EMPTY_TUPLE, dtype=dtype).reshape(0, 0)
                     return np.array(EMPTY_TUPLE, dtype=dtype)
-                else:
-                    return array
+                return array
 
     if dtype.kind == 'O':
         # assume that 1D arrays arrays are arrays of tuples
@@ -1572,13 +1622,17 @@ def _ufunc_set_2d(
         except TypeError:
             values = tuple(result)
 
-        # returns a 1D object array of tuples
+        if is_2d:
+            if len(values) == 0:
+                return np.array(EMPTY_TUPLE, dtype=dtype).reshape(0, 0)
+            return np.array(values, dtype=object)
+
         post = np.empty(len(values), dtype=object)
         post[:] = values
         return post
 
     # from here, we assume we have two 2D arrays
-    if array.ndim != 2 or other.ndim != 2:
+    if not is_2d:
         raise RuntimeError('non-object arrays have to both be 2D')
 
     # number of columns must be the same, as doing row-wise comparison, and determines the length of each row
@@ -1649,7 +1703,6 @@ def setdiff1d(
 def union2d(
         array: np.ndarray,
         other: np.ndarray,
-        *,
         assume_unique: bool=False
         ) -> np.ndarray:
     '''
@@ -1663,7 +1716,6 @@ def union2d(
 def intersect2d(
         array: np.ndarray,
         other: np.ndarray,
-        *,
         assume_unique: bool=False
         ) -> np.ndarray:
     '''
@@ -1677,7 +1729,6 @@ def intersect2d(
 def setdiff2d(
         array: np.ndarray,
         other: np.ndarray,
-        *,
         assume_unique: bool=False
         ) -> np.ndarray:
     '''
@@ -1691,7 +1742,6 @@ def setdiff2d(
 def ufunc_set_iter(
         arrays: tp.Iterable[np.ndarray],
         union: bool = False,
-        *,
         assume_unique: bool=False
         ) -> np.ndarray:
     '''
@@ -1709,7 +1759,7 @@ def ufunc_set_iter(
         ufunc = union1d if union else intersect1d
         ndim = 1
     else: # ndim == 2
-        ufunc = union2d if union else intersect2d #type: ignore
+        ufunc = union2d if union else intersect2d
         ndim = 2
 
     for array in arrays:
@@ -1810,6 +1860,92 @@ def isin(
     result.flags.writeable = False
     return result
 
+#-------------------------------------------------------------------------------
+
+
+def array_from_element_attr(*,
+        array: np.ndarray,
+        attr_name: str,
+        dtype: np.dtype
+        ) -> np.array:
+    '''
+    Handle element-wise attribute acesss on arrays of Python date/datetime objects.
+    '''
+    if array.ndim == 1:
+        post = np.fromiter(
+                (getattr(d, attr_name) for d in array),
+                count=len(array),
+                dtype=dtype,
+                )
+    else:
+        post = np.empty(shape=array.shape, dtype=dtype)
+        for iloc, e in np.ndenumerate(array):
+            post[iloc] = getattr(e, attr_name)
+
+    post.flags.writeable = False
+    return post
+
+
+def array_from_element_method(*,
+        array: np.ndarray,
+        method_name: str,
+        args: tp.Tuple[tp.Any, ...],
+        dtype: np.dtype,
+        pre_insert: tp.Optional[AnyCallable] = None,
+        ) -> np.array:
+    '''
+    Handle element-wise method calling on arrays of Python date/datetime objects.
+
+    Args:
+        pre_insert:
+    '''
+    if dtype == DTYPE_STR:
+        # build into a list first, then construct array to determine size
+        if array.ndim == 1:
+            if pre_insert:
+                proto = [pre_insert(getattr(d, method_name)(*args)) for d in array]
+            else:
+                proto = [getattr(d, method_name)(*args) for d in array]
+        else:
+            proto = [[None for _ in range(array.shape[1])]
+                    for _ in range(array.shape[0])]
+            if pre_insert:
+                for (y, x), e in np.ndenumerate(array):
+                    proto[y][x] = pre_insert(getattr(e, method_name)(*args))
+            else:
+                for (y, x), e in np.ndenumerate(array):
+                    proto[y][x] = getattr(e, method_name)(*args)
+        post = np.array(proto, dtype=dtype)
+
+    else:
+        if array.ndim == 1 and dtype != DTYPE_OBJECT:
+            # NOTE: can I get the method off the clas and pass self
+            if pre_insert:
+                post = np.fromiter(
+                        (pre_insert(getattr(d, method_name)(*args)) for d in array),
+                        count=len(array),
+                        dtype=dtype,
+                        )
+            else:
+                post = np.fromiter(
+                        (getattr(d, method_name)(*args) for d in array),
+                        count=len(array),
+                        dtype=dtype,
+                        )
+        else:
+            post = np.empty(shape=array.shape, dtype=dtype)
+            if pre_insert:
+                for iloc, e in np.ndenumerate(array):
+                    post[iloc] = pre_insert(getattr(e, method_name)(*args))
+            else:
+                for iloc, e in np.ndenumerate(array):
+                    post[iloc] = getattr(e, method_name)(*args)
+
+    post.flags.writeable = False
+    return post
+
+
+
 
 #-------------------------------------------------------------------------------
 
@@ -1926,7 +2062,7 @@ def write_optional_file(
     else: # string IO
         f.write(content)
         f.seek(0)
-    return tp.cast(str, fp)
+    return tp.cast(str, fp) # this migh return None with a StringIO
 
 
 #-------------------------------------------------------------------------------
